      subroutine aerochem_cf(h2o,tempk,press,cwc,cph,ccgh,con,convfac,
     &                       dtaq,dtaer,delcon,scgh,sddm,nfam,nsen,
     &                       ldoipr,ipa_cel,aero_flag,lpig,icig)
      use camxcom
      use chmstry
      use filunit
      use tracer
      use procan
c
c----CAMx v6.30 160408
c
c     AEROCHEM_CF drives the CF 2-section aerosol model in CAMx.
c     It calculates the following chemical transformations:
c        1. Condensible organic gasses to organic aerosol (SOAP or VBS)
c        2. Gaseous sulfate to aerosol sulfate (from gas-phase chem)
c        3. Inorganic aqueous reactions (RADM-AQ approach)
c        4. Inorganic gas-aerosol equilibrium partitioning for the
c           ammonium/nitrate/sulfate/sodium/chloride system (ISORROPIA)
c     The gas species are in ppm
c     The aerosol species are in (ug/m3)
c
c     Copyright 1996 - 2016
c     Ramboll ENVIRON
c
c     Modifications: 
c        1/9/02         Minor bug fixes (units conversions)
c        11/19/02       Incorporated RADM aqueous chemistry and ISORROPIA
c        12/9/02        Incorporated SOAP routine
c        10/24/05       Cloud pH is now passed in/out; Henry's Law values
c                       for SO2 and O3 no longer passed out
c        8/23/06        Removed metastable option in ISOROPIA
c        12/26/06       AQ-CHEM now called each chemistry timestep
c        12/29/06       Revised for the updated SOA scheme
c        01/10/07       Added species mapping for CB05/SAPRC99
c        05/04/07       Added array (wt_save) to keep original total
c                       mass because ISOROPIA overwrites with lower
c                       bound value.
c        05/30/07       Skip call to RAQCHEM under high acid conditions.
c        05/13/08       Modified to skip ISOROPIA for certain PiG calls.
c        01/21/10       Added formation of mineral dust nitrate
c        07/31/13       Modified for new GREASD PiG chemistry 
c        08/23/13       Revised for DDM-PM; replaced hard-wired MW's with wtmol
c        03/18/14       Revised for benzene SOA
c        10/09/14       Added subgrid convective model
c        12/07/14       Revised for VBS
c        01/08/16       Updated for Revised VBS
c
c     Input arguments:
c        h2o                 cell water vapor (ppm)
c        tempk               cell temperature (K)
c        press               cell pressure (mb)
c        cwc                 cloud water content (g/m3)
c        cph                 cloud water pH 
c        ccgh                CGH concentrations (ppm)
c        con                 species concentrations (ppm, ug/m3)
c        convfac             conversion factor: umol/m3 = ppm * convfac
c        dtaq                aqueous chemistry time step (hours)
c        dtaer               general PM chemistry time step (hours)
c        delcon              delta concentrations for source apportionment
c        scgh                CGH sensitivities
c        sddm                DDM-PM sensitivity array
c        nfam                number of DDM parameters
c        nsen                number of model species used in DDM-PM
c        ldoipr              flag to determine whether to do IPR calculation
c        ipa_cel             pointer into PA sub-domain array
c        aero_flag           1 - do only RAQCHEM (with ISOROPIA);
c                            2 - do SOAP, RAQCHEM, ISOROPIA
c        lpig                T - PiG call (do only RAQCHEM)
c        icig                0 - CiG not activated (default to aero_flag)
c                            1 - CiG call (do only RAQCHEM)
c                            2 - Grid call, CiG called prior (skip RAQCHEM)
c
c     Output arguments:
c        con                 species concentrations (ppm, ug/m3)
c        cph                 cloud water pH 
c        delcon              delta concentrations for source apportionment
c        sddm                DDM-PM sensitivity array
c
c     Routines called: 
c        SOAP
c        VBSPART
c        RAQCHEM
c        ISOROPIA
c 
c     Called by: 
c        CIGDRIVE
c        CHEMDRIV 
c
      implicit none
      include 'camx.prm' 
      include 'camx_aero.inc'
      include 'chmdbg.inc'
c
c========================== Process Analysis Begin =======================
c
      logical ldoipr
      integer ipa_cel
      real csulf,chno3,cnh3,cpso4,cpno3,cpnh4
c
c========================== Process Analysis End =======================
c
c========================= DDM Begin ===================================
c
      integer nfam, nsen, ierr, ip
      real scgh(nfam)
      real sddm(nfam,nsen)
c
c========================== DDM End ====================================
c
      real    h2o
      real    tempk
      real    press
      real    cwc
      real    cph
      integer, parameter :: NSET = 5
      real    ccgh(NSET)
      real    con(*)
      real    convfac
      real    dtaq
      real    dtaer
      real    delcon(5,*)
      real    so2
      real    so4
      real    no3
      integer aero_flag
      integer icig
      logical lpig
      logical lsoap
      logical lradm
      logical lisor

      integer ispc,is
      real qwatr,ev,es,rh
      real pres_pa
      real dt_sec
      real cold(MXSPEC), deln2o5
      real cw_kgm3
c
c-----Arrays for SOAP/VBSPART
c
      integer, parameter :: NCG = 7
      integer, parameter :: NVBIN = 4
      integer, parameter :: NOPAIR = MAX( NCG, NSET*NVBIN )
      real soacon(NOPAIR),cg(NOPAIR),csatT(NOPAIR)
      real cnvol(NSET),mwnvol(NSET)
      real apoly(NCG)
c
c-----Arrays for RADM aqueous chemistry
c
      real r_gas(11),r_aer(9),r_old(20)
      real r_1,r_2
c
c-----Variables for ISOROPIA need double precision
c
      real*8 wi(5),wt(5),wt_save(5)
      real*8 rhi,tempi,cntrl(2)
      real*8 gasis(3),aerliq(12),aersld(9),other(6)
      character*15 scasi
c
c-----Variables for mineral dust nitrate
c
      real*8 dno3 ! mineral dust nitrate (mol/m3)
      real fCaCO3, fca0
      parameter (fCaCO3 = 0.065) ! mass fraction of CaCO3 on dust
c 
c-----Entry point 
c
      lsoap = .false.
      lradm = .false.
      lisor = .false.
      if (aero_flag.eq.2) then
        lsoap = .true.
        lisor = .true.        
      endif
      if (cwc.ge.cwmin .and. tempk.gt.tamin) then
        so2 = con(kso2)*1.e-6
        so4 = (con(kpso4)/wtmol(1)/convfac)*1.e-6
        no3 = (con(kpno3)/wtmol(3)/convfac)*1.e-6
        if (so4.lt.5.0e-7 .and. no3.lt.1.5e-6 .and. so2.lt.1.0e-6) then
          lradm = .true.
          lisor = .true.        
        endif
        if (lpig) lisor = .false.
        if (icig.eq.1) lisor = .false.
        if (icig.eq.2) lradm = .false.
      endif

      do ispc = 1,nspec
         cold(ispc) = con(ispc)
      enddo
      con(nspec+1) = 0.
      cph = 5.0
c
c-----Calculate relative humidity
c
      qwatr = 1.e-6*h2o*18./28.8 
      ev = qwatr*press/(qwatr + eps) 
      es = e0*exp((lv/rv)*(1./273. - 1./tempk)) 
      rh = 100.0*amin1(0.99,ev/es)
c
c-----Partitioning of condensable secondary organics between the gas (CG)
c     and aerosol phases (SOA) using the SOAP semi-volatile scheme.
c     CGs are in ppm, SOAs are in ug/m3, SOAP does the units conversion.
c
      if ( .not.lpig ) then
      if ( aeropt.eq.'CF_VBS' ) then

        mwnvol(1) = 172.0
        mwnvol(2) = 172.0
        mwnvol(3) = 278.0
        mwnvol(4) = 278.0
        mwnvol(5) = 205.0

        con(kPAS0) = con(kPAS0) + ccgh(1) * convfac * mwnvol(1)
        con(kPBS0) = con(kPBS0) + ccgh(2) * convfac * mwnvol(2)
        con(kPAP0) = con(kPAP0) + ccgh(3) * convfac * mwnvol(3)
        con(kPCP0) = con(kPCP0) + ccgh(4) * convfac * mwnvol(4)
        con(kPFP0) = con(kPFP0) + ccgh(5) * convfac * mwnvol(5)
        if ( ldoipr ) then
          cipr(IPR_OAERO,ipa_cel,kPAS0) = cipr(IPR_OAERO,ipa_cel,kPAS0)
     &                                  + con(kPAS0)-cold(kPAS0)
          cipr(IPR_OAERO,ipa_cel,kPBS0) = cipr(IPR_OAERO,ipa_cel,kPBS0)
     &                                  + con(kPBS0)-cold(kPBS0)
          cipr(IPR_OAERO,ipa_cel,kPAP0) = cipr(IPR_OAERO,ipa_cel,kPAP0)
     &                                  + con(kPAP0)-cold(kPAP0)
          cipr(IPR_OAERO,ipa_cel,kPCP0) = cipr(IPR_OAERO,ipa_cel,kPCP0)
     &                                  + con(kPCP0)-cold(kPCP0)
          cipr(IPR_OAERO,ipa_cel,kPFP0) = cipr(IPR_OAERO,ipa_cel,kPFP0)
     &                                  + con(kPFP0)-cold(kPFP0)
        endif

        if (aero_flag.eq.2) then
          cg(1)  = con(kVAS1)
          cg(2)  = con(kVAS2)
          cg(3)  = con(kVAS3)
          cg(4)  = con(kVAS4)
          cg(5)  = con(kVBS1)
          cg(6)  = con(kVBS2)
          cg(7)  = con(kVBS3)
          cg(8)  = con(kVBS4)
          cg(9)  = con(kVAP1)
          cg(10) = con(kVAP2)
          cg(11) = con(kVAP3)
          cg(12) = con(kVAP4)
          cg(13) = con(kVCP1)
          cg(14) = con(kVCP2)
          cg(15) = con(kVCP3)
          cg(16) = con(kVCP4)
          cg(17) = con(kVFP1)
          cg(18) = con(kVFP2)
          cg(19) = con(kVFP3)
          cg(20) = con(kVFP4)
          soacon(1)  = con(kPAS1)
          soacon(2)  = con(kPAS2)
          soacon(3)  = con(kPAS3)
          soacon(4)  = con(kPAS4)
          soacon(5)  = con(kPBS1)
          soacon(6)  = con(kPBS2)
          soacon(7)  = con(kPBS3)
          soacon(8)  = con(kPBS4)
          soacon(9)  = con(kPAP1)
          soacon(10) = con(kPAP2)
          soacon(11) = con(kPAP3)
          soacon(12) = con(kPAP4)
          soacon(13) = con(kPCP1)
          soacon(14) = con(kPCP2)
          soacon(15) = con(kPCP3)
          soacon(16) = con(kPCP4)
          soacon(17) = con(kPFP1)
          soacon(18) = con(kPFP2)
          soacon(19) = con(kPFP3)
          soacon(20) = con(kPFP4)
          cnvol(1)   = con(kPAS0)
          cnvol(2)   = con(kPBS0)
          cnvol(3)   = con(kPAP0)
          cnvol(4)   = con(kPCP0)
          cnvol(5)   = con(kPFP0)
          call vbspart(NSET*NVBIN,NSET,
     &                 soacon(:NSET*NVBIN),cg(:NSET*NVBIN),mwnvol,cnvol,
     &                 tempk,convfac,iout,igrdchm,ichm,jchm,kchm,.TRUE.,
     &                 csatT(:NSET*NVBIN))
          con(kVAS1) = amax1(cg(1) ,bdnl(kVAS1))
          con(kVAS2) = amax1(cg(2) ,bdnl(kVAS2))
          con(kVAS3) = amax1(cg(3) ,bdnl(kVAS3))
          con(kVAS4) = amax1(cg(4) ,bdnl(kVAS4))
          con(kVBS1) = amax1(cg(5) ,bdnl(kVBS1))
          con(kVBS2) = amax1(cg(6) ,bdnl(kVBS2))
          con(kVBS3) = amax1(cg(7) ,bdnl(kVBS3))
          con(kVBS4) = amax1(cg(8) ,bdnl(kVBS4))
          con(kVAP1) = amax1(cg(9) ,bdnl(kVAP1))
          con(kVAP2) = amax1(cg(10),bdnl(kVAP2))
          con(kVAP3) = amax1(cg(11),bdnl(kVAP3))
          con(kVAP4) = amax1(cg(12),bdnl(kVAP4))
          con(kVCP1) = amax1(cg(13),bdnl(kVCP1))
          con(kVCP2) = amax1(cg(14),bdnl(kVCP2))
          con(kVCP3) = amax1(cg(15),bdnl(kVCP3))
          con(kVCP4) = amax1(cg(16),bdnl(kVCP4))
          con(kVFP1) = amax1(cg(17),bdnl(kVFP1))
          con(kVFP2) = amax1(cg(18),bdnl(kVFP2))
          con(kVFP3) = amax1(cg(19),bdnl(kVFP3))
          con(kVFP4) = amax1(cg(20),bdnl(kVFP4))
          con(kPAS1) = amax1(soacon(1) ,bdnl(kPAS1))
          con(kPAS2) = amax1(soacon(2) ,bdnl(kPAS2))
          con(kPAS3) = amax1(soacon(3) ,bdnl(kPAS3))
          con(kPAS4) = amax1(soacon(4) ,bdnl(kPAS4))
          con(kPBS1) = amax1(soacon(5) ,bdnl(kPBS1))
          con(kPBS2) = amax1(soacon(6) ,bdnl(kPBS2))
          con(kPBS3) = amax1(soacon(7) ,bdnl(kPBS3))
          con(kPBS4) = amax1(soacon(8) ,bdnl(kPBS4))
          con(kPAP1) = amax1(soacon(9) ,bdnl(kPAP1))
          con(kPAP2) = amax1(soacon(10),bdnl(kPAP2))
          con(kPAP3) = amax1(soacon(11),bdnl(kPAP3))
          con(kPAP4) = amax1(soacon(12),bdnl(kPAP4))
          con(kPCP1) = amax1(soacon(13),bdnl(kPCP1))
          con(kPCP2) = amax1(soacon(14),bdnl(kPCP2))
          con(kPCP3) = amax1(soacon(15),bdnl(kPCP3))
          con(kPCP4) = amax1(soacon(16),bdnl(kPCP4))
          con(kPFP1) = amax1(soacon(17),bdnl(kPFP1))
          con(kPFP2) = amax1(soacon(18),bdnl(kPFP2))
          con(kPFP3) = amax1(soacon(19),bdnl(kPFP3))
          con(kPFP4) = amax1(soacon(20),bdnl(kPFP4))
c
c========================= Process Analysis Begin ======================
c
          if ( ldoipr ) then
            cipr(IPR_OAERO,ipa_cel,kVAS1)=cipr(IPR_OAERO,ipa_cel,kVAS1)
     &                               + (con(kVAS1)-cold(kVAS1))*convfac
            cipr(IPR_OAERO,ipa_cel,kVAS2)=cipr(IPR_OAERO,ipa_cel,kVAS2)
     &                               + (con(kVAS2)-cold(kVAS2))*convfac
            cipr(IPR_OAERO,ipa_cel,kVAS3)=cipr(IPR_OAERO,ipa_cel,kVAS3)
     &                               + (con(kVAS3)-cold(kVAS3))*convfac
            cipr(IPR_OAERO,ipa_cel,kVAS4)=cipr(IPR_OAERO,ipa_cel,kVAS4)
     &                               + (con(kVAS4)-cold(kVAS4))*convfac
            cipr(IPR_OAERO,ipa_cel,kVBS1)=cipr(IPR_OAERO,ipa_cel,kVBS1)
     &                               + (con(kVBS1)-cold(kVBS1))*convfac
            cipr(IPR_OAERO,ipa_cel,kVBS2)=cipr(IPR_OAERO,ipa_cel,kVBS2)
     &                               + (con(kVBS2)-cold(kVBS2))*convfac
            cipr(IPR_OAERO,ipa_cel,kVBS3)=cipr(IPR_OAERO,ipa_cel,kVBS3)
     &                               + (con(kVBS3)-cold(kVBS3))*convfac
            cipr(IPR_OAERO,ipa_cel,kVBS4)=cipr(IPR_OAERO,ipa_cel,kVBS4)
     &                               + (con(kVBS4)-cold(kVBS4))*convfac
            cipr(IPR_OAERO,ipa_cel,kVAP1)=cipr(IPR_OAERO,ipa_cel,kVAP1)
     &                               + (con(kVAP1)-cold(kVAP1))*convfac
            cipr(IPR_OAERO,ipa_cel,kVAP2)=cipr(IPR_OAERO,ipa_cel,kVAP2)
     &                               + (con(kVAP2)-cold(kVAP2))*convfac
            cipr(IPR_OAERO,ipa_cel,kVAP3)=cipr(IPR_OAERO,ipa_cel,kVAP3)
     &                               + (con(kVAP3)-cold(kVAP3))*convfac
            cipr(IPR_OAERO,ipa_cel,kVAP4)=cipr(IPR_OAERO,ipa_cel,kVAP4)
     &                               + (con(kVAP4)-cold(kVAP4))*convfac
            cipr(IPR_OAERO,ipa_cel,kVCP1)=cipr(IPR_OAERO,ipa_cel,kVCP1)
     &                               + (con(kVCP1)-cold(kVCP1))*convfac
            cipr(IPR_OAERO,ipa_cel,kVCP2)=cipr(IPR_OAERO,ipa_cel,kVCP2)
     &                               + (con(kVCP2)-cold(kVCP2))*convfac
            cipr(IPR_OAERO,ipa_cel,kVCP3)=cipr(IPR_OAERO,ipa_cel,kVCP3)
     &                               + (con(kVCP3)-cold(kVCP3))*convfac
            cipr(IPR_OAERO,ipa_cel,kVCP4)=cipr(IPR_OAERO,ipa_cel,kVCP4)
     &                               + (con(kVCP4)-cold(kVCP4))*convfac
            cipr(IPR_OAERO,ipa_cel,kVFP1)=cipr(IPR_OAERO,ipa_cel,kVFP1)
     &                               + (con(kVFP1)-cold(kVFP1))*convfac
            cipr(IPR_OAERO,ipa_cel,kVFP2)=cipr(IPR_OAERO,ipa_cel,kVFP2)
     &                               + (con(kVFP2)-cold(kVFP2))*convfac
            cipr(IPR_OAERO,ipa_cel,kVFP3)=cipr(IPR_OAERO,ipa_cel,kVFP3)
     &                               + (con(kVFP3)-cold(kVFP3))*convfac
            cipr(IPR_OAERO,ipa_cel,kVFP4)=cipr(IPR_OAERO,ipa_cel,kVFP4)
     &                               + (con(kVFP4)-cold(kVFP4))*convfac
            cipr(IPR_OAERO,ipa_cel,kPAS1)=cipr(IPR_OAERO,ipa_cel,kPAS1)
     &                                  + con(kPAS1)-cold(kPAS1)
            cipr(IPR_OAERO,ipa_cel,kPAS2)=cipr(IPR_OAERO,ipa_cel,kPAS2)
     &                                  + con(kPAS2)-cold(kPAS2)
            cipr(IPR_OAERO,ipa_cel,kPAS3)=cipr(IPR_OAERO,ipa_cel,kPAS3)
     &                                  + con(kPAS3)-cold(kPAS3)
            cipr(IPR_OAERO,ipa_cel,kPAS4)=cipr(IPR_OAERO,ipa_cel,kPAS4)
     &                                  + con(kPAS4)-cold(kPAS4)
            cipr(IPR_OAERO,ipa_cel,kPBS1)=cipr(IPR_OAERO,ipa_cel,kPBS1)
     &                                  + con(kPBS1)-cold(kPBS1)
            cipr(IPR_OAERO,ipa_cel,kPBS2)=cipr(IPR_OAERO,ipa_cel,kPBS2)
     &                                  + con(kPBS2)-cold(kPBS2)
            cipr(IPR_OAERO,ipa_cel,kPBS3)=cipr(IPR_OAERO,ipa_cel,kPBS3)
     &                                  + con(kPBS3)-cold(kPBS3)
            cipr(IPR_OAERO,ipa_cel,kPBS4)=cipr(IPR_OAERO,ipa_cel,kPBS4)
     &                                  + con(kPBS4)-cold(kPBS4)
            cipr(IPR_OAERO,ipa_cel,kPAP1)=cipr(IPR_OAERO,ipa_cel,kPAP1)
     &                                  + con(kPAP1)-cold(kPAP1)
            cipr(IPR_OAERO,ipa_cel,kPAP2)=cipr(IPR_OAERO,ipa_cel,kPAP2)
     &                                  + con(kPAP2)-cold(kPAP2)
            cipr(IPR_OAERO,ipa_cel,kPAP3)=cipr(IPR_OAERO,ipa_cel,kPAP3)
     &                                  + con(kPAP3)-cold(kPAP3)
            cipr(IPR_OAERO,ipa_cel,kPAP4)=cipr(IPR_OAERO,ipa_cel,kPAP4)
     &                                  + con(kPAP4)-cold(kPAP4)
            cipr(IPR_OAERO,ipa_cel,kPCP1)=cipr(IPR_OAERO,ipa_cel,kPCP1)
     &                                  + con(kPCP1)-cold(kPCP1)
            cipr(IPR_OAERO,ipa_cel,kPCP2)=cipr(IPR_OAERO,ipa_cel,kPCP2)
     &                                  + con(kPCP2)-cold(kPCP2)
            cipr(IPR_OAERO,ipa_cel,kPCP3)=cipr(IPR_OAERO,ipa_cel,kPCP3)
     &                                  + con(kPCP3)-cold(kPCP3)
            cipr(IPR_OAERO,ipa_cel,kPCP4)=cipr(IPR_OAERO,ipa_cel,kPCP4)
     &                                  + con(kPCP4)-cold(kPCP4)
            cipr(IPR_OAERO,ipa_cel,kPFP1)=cipr(IPR_OAERO,ipa_cel,kPFP1)
     &                                  + con(kPFP1)-cold(kPFP1)
            cipr(IPR_OAERO,ipa_cel,kPFP2)=cipr(IPR_OAERO,ipa_cel,kPFP2)
     &                                  + con(kPFP2)-cold(kPFP2)
            cipr(IPR_OAERO,ipa_cel,kPFP3)=cipr(IPR_OAERO,ipa_cel,kPFP3)
     &                                  + con(kPFP3)-cold(kPFP3)
            cipr(IPR_OAERO,ipa_cel,kPFP4)=cipr(IPR_OAERO,ipa_cel,kPFP4)
     &                                  + con(kPFP4)-cold(kPFP4)
            csulf = con(ksulf)
            chno3 = con(khno3)
            cnh3  = con(knh3 )
            cpso4 = con(kpso4)
            cpno3 = con(kpno3)
            cpnh4 = con(kpnh4)
          endif
c
c========================== Process Analysis End =======================
c
        endif ! call VBSPART?

      else

        mwnvol(1) = 150.0

        con(ksoah) = con(ksoah) + ccgh(1) * convfac * mwnvol(1)
        if ( ldoipr ) cipr(IPR_OAERO,ipa_cel,ksoah) =
     &                cipr(IPR_OAERO,ipa_cel,ksoah) + con(ksoah)
     &                                              - cold(ksoah)
c
c========================= DDM Begin ===================================
c
        if (lddm) then
          do ip = 1, nfam
            sddm(ip,jdpSOAH) = sddm(ip,jdpSOAH) 
     &                       + scgh(ip)*convfac*mwnvol(1)
          enddo
        endif
c
c========================== DDM End ====================================
c
        if (aero_flag.eq.2) then
          cg(1)  = con(kcg1)
          cg(2)  = con(kcg2)
          cg(3)  = con(kcg3)
          cg(4)  = con(kcg4)
          cg(5)  = con(kcg5)
          cg(6)  = con(kcg6)
          cg(7)  = con(kcg7)
          soacon(1) = con(ksoa1)
          soacon(2) = con(ksoa2)
          soacon(3) = con(ksoa3)
          soacon(4) = con(ksoa4)
          soacon(5) = con(ksoa5)
          soacon(6) = con(ksoa6)
          soacon(7) = con(ksoa7)
          cnvol(1)  = con(ksoah)
          cnvol(2)  = con(ksopa)
          cnvol(3)  = con(ksopb)
          cnvol(4)  = con(kpoa)
c
c========================= DDM Begin ===================================
c
          if (lddm) then
            call DDMSOAINI (convfac,sddm,nfam,nsen)
          endif
c
c========================== DDM End ====================================
c
          call soap(NCG,soacon(:NCG),cg(:NCG),
     &              mwnvol(1),cnvol(1),cnvol(2),cnvol(3),cnvol(4),
     &              tempk,convfac,dtaer,iout,igrdchm,ichm,jchm,kchm,
     &              .TRUE.,apoly,csatT(:NCG))
c
c========================= DDM Begin ===================================
c
          if (lddm) then
            call DDMSOAP (dtaer,convfac,sddm,nfam,nsen,ierr,iout)
            if (ierr.lt.0) then
              write(iout,'(a,4i4)') ' igrd,i,j,k = ',
     &                                          igrdchm,ichm,jchm,kchm
              write(iout,*) ' T,DT = ',tempk,dtaer
              write(iout,*) ' CONVFAC = ',convfac
              write(iout,*) ' CG = ',con(kcg1),con(kcg2),con(kcg3),
     &                               con(kcg4),con(kcg5),con(kcg6),
     &                               con(kcg7)
              write(iout,*) ' SOA = ',con(ksoa1),con(ksoa2),con(ksoa3),
     &                                con(ksoa4),con(ksoa5),con(ksoa6),
     &                                con(ksoa7),con(ksoah),con(ksopa),
     &                                con(ksopb)
              write(iout,*) ' POA = ',con(kpoa)
              do ip = 1, nfam
                write(iout,*) ' SI(',ip,') = ',
     &                                 (sddm(ip,jdpCG1+is-1),is=1,NCG),
     &                                (sddm(ip,jdpSOA1+is-1),is=1,NCG),
     &              sddm(ip,jdpSOAH),sddm(ip,jdpSOPA),sddm(ip,jdpSOPB),
     &                                                 sddm(ip,jdpPOA)
              enddo
              call camxerr()
            endif
          endif
c
c========================== DDM End ====================================
c
          con(kcg1)  = amax1(cg(1),bdnl(kcg1))
          con(kcg2)  = amax1(cg(2),bdnl(kcg2))
          con(kcg3)  = amax1(cg(3),bdnl(kcg3))
          con(kcg4)  = amax1(cg(4),bdnl(kcg4))
          con(kcg5)  = amax1(cg(5),bdnl(kcg5))
          con(kcg6)  = amax1(cg(6),bdnl(kcg6))
          con(kcg7)  = amax1(cg(7),bdnl(kcg7))
          con(ksoa1) = amax1(soacon(1),bdnl(ksoa1))
          con(ksoa2) = amax1(soacon(2),bdnl(ksoa2))
          con(ksoa3) = amax1(soacon(3),bdnl(ksoa3))
          con(ksoa4) = amax1(soacon(4),bdnl(ksoa4))
          con(ksoa5) = amax1(soacon(5),bdnl(ksoa5))
          con(ksoa6) = amax1(soacon(6),bdnl(ksoa6))
          con(ksoa7) = amax1(soacon(7),bdnl(ksoa7))
          con(ksopa) = amax1(cnvol(2),bdnl(ksopa))
          con(ksopb) = amax1(cnvol(3),bdnl(ksopb))
c
c========================= Process Analysis Begin ======================
c
          if ( ldoipr ) then
            cipr(IPR_OAERO,ipa_cel,kcg1) = cipr(IPR_OAERO,ipa_cel,kcg1)
     &                                 + (con(kcg1)-cold(kcg1))*convfac
            cipr(IPR_OAERO,ipa_cel,kcg2) = cipr(IPR_OAERO,ipa_cel,kcg2)
     &                                 + (con(kcg2)-cold(kcg2))*convfac
            cipr(IPR_OAERO,ipa_cel,kcg3) = cipr(IPR_OAERO,ipa_cel,kcg3)
     &                                 + (con(kcg3)-cold(kcg3))*convfac
            cipr(IPR_OAERO,ipa_cel,kcg4) = cipr(IPR_OAERO,ipa_cel,kcg4)
     &                                 + (con(kcg4)-cold(kcg4))*convfac
            cipr(IPR_OAERO,ipa_cel,kcg5) = cipr(IPR_OAERO,ipa_cel,kcg5)
     &                                 + (con(kcg5)-cold(kcg5))*convfac
            cipr(IPR_OAERO,ipa_cel,kcg6) = cipr(IPR_OAERO,ipa_cel,kcg6)
     &                                 + (con(kcg6)-cold(kcg6))*convfac
            cipr(IPR_OAERO,ipa_cel,kcg7) = cipr(IPR_OAERO,ipa_cel,kcg7)
     &                                 + (con(kcg7)-cold(kcg7))*convfac
            cipr(IPR_OAERO,ipa_cel,ksoa1)=cipr(IPR_OAERO,ipa_cel,ksoa1)
     &                                 + con(ksoa1)-cold(ksoa1)
            cipr(IPR_OAERO,ipa_cel,ksoa2)=cipr(IPR_OAERO,ipa_cel,ksoa2)
     &                                 + con(ksoa2)-cold(ksoa2)
            cipr(IPR_OAERO,ipa_cel,ksoa3)=cipr(IPR_OAERO,ipa_cel,ksoa3)
     &                                 + con(ksoa3)-cold(ksoa3)
            cipr(IPR_OAERO,ipa_cel,ksoa4)=cipr(IPR_OAERO,ipa_cel,ksoa4)
     &                                 + con(ksoa4)-cold(ksoa4)
            cipr(IPR_OAERO,ipa_cel,ksoa5)=cipr(IPR_OAERO,ipa_cel,ksoa5)
     &                                 + con(ksoa5)-cold(ksoa5)
            cipr(IPR_OAERO,ipa_cel,ksoa6)=cipr(IPR_OAERO,ipa_cel,ksoa6)
     &                                 + con(ksoa6)-cold(ksoa6)
            cipr(IPR_OAERO,ipa_cel,ksoa7)=cipr(IPR_OAERO,ipa_cel,ksoa7)
     &                                 + con(ksoa7)-cold(ksoa7)
            cipr(IPR_OAERO,ipa_cel,ksoah)=cipr(IPR_OAERO,ipa_cel,ksoah)
     &                                 + con(ksoah)-cold(ksoah)
            cipr(IPR_OAERO,ipa_cel,ksopa)=cipr(IPR_OAERO,ipa_cel,ksopa)
     &                                 + con(ksopa)-cold(ksopa)
            cipr(IPR_OAERO,ipa_cel,ksopb)=cipr(IPR_OAERO,ipa_cel,ksopb)
     &                                 + con(ksopb)-cold(ksopb)
            csulf = con(ksulf)
            chno3 = con(khno3)
            cnh3  = con(knh3 )
            cpso4 = con(kpso4)
            cpno3 = con(kpno3)
            cpnh4 = con(kpnh4)
          endif
c
c========================== Process Analysis End =======================
c
        endif ! call SOAP?

      endif ! CF or CF_VBS
      endif ! .not. PiG
c
c-----Do RADM aqueous chemistry if CWC is above threshold
c     All conc units must be mol/mol (mixing ratio)
c
      if (lradm) then
        pres_pa = 100.*press
        dt_sec = dtaq*3600.
        cw_kgm3 = cwc/1000.
c
        r_gas(1)  = con(kso2)*1.e-6
        r_gas(2)  = con(khno3)*1.e-6
        r_gas(3)  = con(kn2o5)*1.e-6
        r_gas(4)  = co2*1.e-6
        r_gas(5)  = con(knh3)*1.e-6
        r_gas(6)  = con(khpo_c)*1.e-6
        r_gas(7)  = con(ko3)*1.e-6
        if (kfoa_c.lt.nspec+1) then
          r_gas(8) = con(kfoa_c)*1.e-6
        else
          r_gas(8) = foa*1.e-6
        endif
        if (kmhp_c.lt.nspec+1 .or. kohp_c.lt.nspec+1) then
          r_1 = con(kmhp_c) + con(kohp_c)
          r_gas(9) = r_1*1.e-6
          r_1 = con(kmhp_c) / r_1
        else
          r_gas(9) = mhp*1.e-6
          r_1 = 0.0
        endif
        if (kpaa_c.lt.nspec+1 .or. kopa_c.lt.nspec+1) then
          r_2 = con(kpaa_c) + con(kopa_c)
          r_gas(10) = r_2*1.e-6
          r_2 = con(kpaa_c) / r_2
        else
          r_gas(10) = paa*1.e-6
          r_2 = 0.0
        endif
        r_gas(11) = con(ksulf)*1.e-6
c
        r_aer(1)  = (con(kpso4)/wtmol(1)/convfac)*1.e-6
        r_aer(2)  = (con(kpnh4)/wtmol(2)/convfac)*1.e-6
        r_aer(3)  = (con(kpno3)/wtmol(3)/convfac)*1.e-6
        r_aer(4)  = (fCaCO3*con(kFCRS)/wtmol(7)/convfac)*1.e-6
        r_aer(5)  = (mgco3/wtmol(8)/convfac)*1.e-6
        if (kna.eq.nspec+1) then
          r_aer(6) = (nacl/(wtmol(5)+wtmol(4))/convfac)*1.e-6
        else
          if (con(kna).gt.con(kpcl)) then
            r_aer(6) = (con(kpcl)/wtmol(4)/convfac)*1.e-6
          else
            r_aer(6) = (con(kna)/wtmol(5)/convfac)*1.e-6
          endif
        endif
        r_aer(7)  = (a3fe/wtmol(9)/convfac)*1.e-6
        r_aer(8)  = (b2mn/wtmol(10)/convfac)*1.e-6
        r_aer(9)  = (potcl/(wtmol(6)+wtmol(4))/convfac)*1.e-6
c
c========================= DDM Begin ===================================
c
        if (lddm) then
          do is = 1, 11
            r_old(is) = r_gas(is)
          enddo
          do is = 1,9
            r_old(is+11) = r_aer(is)
          enddo
          call DDMRADINI (con(kso2),con(kna),con(kpcl),fCaCO3,convfac,
     &                                    wtmol,sddm,nfam,nsen,NWTMOL)
        endif
c
c========================== DDM End ====================================
c
        call raqchem(tempk,pres_pa,dt_sec,cw_kgm3,r_gas,r_aer,cph,
     &               idiag,iout,igrdchm,ichm,jchm,kchm)
c
c========================= DDM Begin ===================================
c
        if (lddm) then
          call DDMRADM (r_1,r_gas(9)/r_old(9),r_2,r_gas(10)/r_old(10),
     &                  convfac,wtmol,sddm,nfam,nsen,NWTMOL,ierr,iout)
          if (ierr.lt.0) then
            write(iout,'(a,4i4)') ' igrd,i,j,k = ',
     &                                          igrdchm,ichm,jchm,kchm
            write(iout,*) ' T,P,CW,DT = ',tempk,pres_pa,cw_kgm3,dt_sec
            write(iout,*) ' CONVFAC = ',convfac
            write(iout,*) ' r1,r11,r2,r22 = ',r_1,r_gas(9)/r_old(9)
     &                                       ,r_2,r_gas(10)/r_old(10)
            write(iout,*) ' R_GAS = ',(r_old(is),is=1,11)
            write(iout,*) ' R_AER = ',(r_old(is+11),is=1,9)
            do ip = 1, nfam
              if (con(kna).gt.con(kpcl)) then
                r_1 = sddm(ip,jdpPCL )/wtmol(4)/convfac *1.e-6
              else
                r_1 = sddm(ip,jdpNA  )/wtmol(5)/convfac *1.e-6
              endif
              write(iout,*) ' SI(',ip,') = ',
     &      (sddm(ip,jdpNH3 )+sddm(ip,jdpPNH4)/wtmol(2)/convfac)*1.e-6,
     &      (sddm(ip,jdpHNO3)+sddm(ip,jdpN2O5)*2.0
     &                       +sddm(ip,jdpPNO3)/wtmol(3)/convfac)*1.e-6,
     &             sddm(ip,jdpFOA )*1.e-6, 0.0, sddm(ip,jdpSO2 )*1.e-6,
     &      (sddm(ip,jdpSULF)+sddm(ip,jdpPSO4)/wtmol(1)/convfac)*1.e-6,
     &                  sddm(ip,jdpH2O2)*1.e-6, sddm(ip,jdpO3  )*1.e-6,
     &                   ( sddm(ip,jdpMHP ) + sddm(ip,jdpOHP ) )*1.e-6,
     &                   ( sddm(ip,jdpPAA ) + sddm(ip,jdpOPA ) )*1.e-6,
     &              r_1,fCaCO3*sddm(ip,jdpFCRS)/wtmol(7)/convfac*1.e-6
            enddo
            call camxerr()
          endif
        endif
c
c========================== DDM End ====================================
c
        con(kso2)  = amax1(r_gas(1) *1.e6,bdnl(kso2))     ! SO2 (ppm)
        con(khno3) = amax1(r_gas(2) *1.e6,bdnl(khno3))    ! HNO3 (ppm)
        con(kn2o5) = amax1(r_gas(3) *1.e6,bdnl(kn2o5))    ! N2O5 gas (ppm)
        con(knh3)  = amax1(r_gas(5) *1.e6,bdnl(knh3))     ! NH3 (ppm)
        con(khpo_c)= amax1(r_gas(6) *1.e6,bdnl(khpo_c))   ! H2O2 (ppm)
        con(ko3)   = amax1(r_gas(7) *1.e6,bdnl(ko3))      ! O3 (ppm)
cbk        con(kfoa_c)= amax1(r_gas(8) *1.e6,bdnl(kfoa_c)) ! not changed by RADM
        con(kmhp_c)= amax1(r_gas(9) *1.e6*r_1,bdnl(kmhp_c))
        con(kohp_c)= amax1(r_gas(9) *1.e6*(1.-r_1),bdnl(kohp_c))
        con(kpaa_c)= amax1(r_gas(10)*1.e6*r_2,bdnl(kpaa_c))
        con(kopa_c)= amax1(r_gas(10)*1.e6*(1.-r_2),bdnl(kopa_c))
        con(ksulf) = amax1(r_gas(11)*1.e6,bdnl(ksulf))    ! H2SO4 (ppm)
c
        con(kpso4) = amax1(r_aer(1)*convfac*96.*1.e6,bdnl(kpso4)) ! PSO4 (ug/m3)
        con(kpnh4) = amax1(r_aer(2)*convfac*18.*1.e6,bdnl(kpnh4)) ! PNH4 (ug/m3)
        con(kpno3) = amax1(r_aer(3)*convfac*62.*1.e6,bdnl(kpno3)) ! PNO3 (ug/m3)
c
        con(nspec+1) = 0.
c
c========================= Process Analysis Begin ======================
c
        if ( ldoipr ) then
          cipr(IPR_AQCHEM,ipa_cel,kso2 )=cipr(IPR_AQCHEM,ipa_cel,kso2 )
     &                                +(con(kso2 )-cold(kso2 ))*convfac
          cipr(IPR_AQCHEM,ipa_cel,khno3)=cipr(IPR_AQCHEM,ipa_cel,khno3)
     &                                +(con(khno3)-cold(khno3))*convfac
          cipr(IPR_AQCHEM,ipa_cel,kn2o5)=cipr(IPR_AQCHEM,ipa_cel,kn2o5)
     &                                +(con(kn2o5)-cold(kn2o5))*convfac
          cipr(IPR_AQCHEM,ipa_cel,knh3 )=cipr(IPR_AQCHEM,ipa_cel,knh3 )
     &                                +(con(knh3 )-cold(knh3 ))*convfac
          cipr(IPR_AQCHEM,ipa_cel,khpo_c)
     &                                 =cipr(IPR_AQCHEM,ipa_cel,khpo_c)
     &                              +(con(khpo_c)-cold(khpo_c))*convfac
          cipr(IPR_AQCHEM,ipa_cel,ko3  )=cipr(IPR_AQCHEM,ipa_cel,ko3  )
     &                                +(con(ko3  )-cold(ko3  ))*convfac
cbk          if (kfoa_c.lt.nspec+1) cipr(IPR_AQCHEM,ipa_cel,kfoa_c) ! not changed by RADM
cbk     &                                 =cipr(IPR_AQCHEM,ipa_cel,kfoa_c)
cbk     &                              +(con(kfoa_c)-cold(kfoa_c))*convfac
          if (kmhp_c.lt.nspec+1) cipr(IPR_AQCHEM,ipa_cel,kmhp_c)
     &                                 =cipr(IPR_AQCHEM,ipa_cel,kmhp_c)
     &                              +(con(kmhp_c)-cold(kmhp_c))*convfac
          if (kohp_c.lt.nspec+1) cipr(IPR_AQCHEM,ipa_cel,kohp_c)
     &                                 =cipr(IPR_AQCHEM,ipa_cel,kohp_c)
     &                              +(con(kohp_c)-cold(kohp_c))*convfac
          if (kpaa_c.lt.nspec+1) cipr(IPR_AQCHEM,ipa_cel,kpaa_c)
     &                                 =cipr(IPR_AQCHEM,ipa_cel,kpaa_c)
     &                              +(con(kpaa_c)-cold(kpaa_c))*convfac
          if (kopa_c.lt.nspec+1) cipr(IPR_AQCHEM,ipa_cel,kopa_c)
     &                                 =cipr(IPR_AQCHEM,ipa_cel,kopa_c)
     &                              +(con(kopa_c)-cold(kopa_c))*convfac
          cipr(IPR_AQCHEM,ipa_cel,ksulf)=cipr(IPR_AQCHEM,ipa_cel,ksulf)
     &                                +(con(ksulf)-cold(ksulf))*convfac
          cipr(IPR_AQCHEM,ipa_cel,kpso4)=cipr(IPR_AQCHEM,ipa_cel,kpso4)
     &                                  +con(kpso4)-cold(kpso4)
          cipr(IPR_AQCHEM,ipa_cel,kpnh4)=cipr(IPR_AQCHEM,ipa_cel,kpnh4)
     &                                  +con(kpnh4)-cold(kpnh4)
          cipr(IPR_AQCHEM,ipa_cel,kpno3)=cipr(IPR_AQCHEM,ipa_cel,kpno3)
     &                                  +con(kpno3)-cold(kpno3)
          csulf = con(ksulf)
          chno3 = con(khno3)
          cnh3  = con(knh3 )
          cpso4 = con(kpso4)
          cpno3 = con(kpno3)
          cpnh4 = con(kpnh4)
        endif
c
c========================== Process Analysis End =======================
c
      endif ! call RADM-AQ?
c
c======================== Source Apportion Begin =======================
c
      if( ltrace ) then
         do ispc=1,ngas
           delcon(2,ispc) = delcon(2,ispc) +
     &                     AMAX1(bdnl(ispc),con(ispc))*convfac
         enddo
         do ispc=ngas+1,nspec
           delcon(2,ispc) = delcon(2,ispc) +
     &                               AMAX1(bdnl(ispc),con(ispc))
         enddo
         deln2o5 = delcon(2,kn2o5) - delcon(1,kn2o5)
         delcon(4,khno3) = delcon(4,khno3) - deln2o5 * 2.0
         delcon(4,kn2o5) = delcon(4,kn2o5) + deln2o5
         if (aeropt.eq.'CF' .and. 
     &       aero_flag.eq.2 .and. .not.lpig .and. icig.eq.0) then
           delcon(5,ksoa1) = -apoly(1)
           delcon(5,ksoa2) = -apoly(2)
           delcon(5,ksoa3) = -apoly(3)
           delcon(5,ksoa4) = -apoly(4)
           delcon(5,ksoa5) = -apoly(5)
           delcon(5,ksoa6) = -apoly(6)
           delcon(5,ksoa7) = -apoly(7)
         endif
      endif
c
c======================== Source Apportion End =======================
c
c
c-----Inorganic aerosol equilibrium chemistry with ISOROPIA
c     Convert conc units to mol/m3 (double precision)
c
      if (lisor) then
        rhi = DBLE(rh/100.)
        tempi = DBLE(tempk)
        cntrl(1) = 0.d0                   ! 0 = forward problem
        cntrl(2) = 0.d0                   ! 0 = solids and liquid allowed
        if (kna.eq.nspec+1) then
          wi(1) = DBLE(nacl/(wtmol(5)+wtmol(4))*1.e-6)                 ! total sodium
          wi(5) = DBLE(con(khcl)*convfac*1.e-6) + wi(1)                ! total chloride
        else
          wi(1) = DBLE(con(kna)/wtmol(5)*1.e-6)                        ! total sodium
          wi(5) = DBLE((con(khcl)*convfac + con(kpcl)/wtmol(4))*1.e-6) ! total chloride
        endif
        wi(2) = DBLE((con(ksulf)*convfac + con(kpso4)/wtmol(1))*1.e-6) ! total sulfate
        wi(3) = DBLE((con(knh3) *convfac + con(kpnh4)/wtmol(2))*1.e-6) ! total ammonium
        wi(4) = DBLE((con(khno3)*convfac + con(kpno3)/wtmol(3))*1.e-6) ! total nitrate

        dno3 = DBLE(fCaCO3*con(kFCRS)/wtmol(7)*1.e-6)            ! dust nitrate
        if (dno3.lt.wi(4)) then
          wi(4) = wi(4) - dno3                                   ! exclude mineral dust nitrate
          fca0  = fCaCO3
        else
          con(khno3) = bdnl(khno3)                               ! all nitrate is
          con(kpno3) = amax1( (SNGL(wi(4))*1.e6 - con(khno3)*convfac)  ! in dust particles
     &                                  * wtmol(3), bdnl(kpno3) )
          wi(4) = 0.d0
          fca0  = -999.
        endif
c
        do is = 1, 5
          wt_save(is) = wi(is)
        enddo
c
c========================= DDM Begin ===================================
c
        if (lddm) then
          call DDMISRINI (fca0,convfac,wtmol,sddm,nfam,nsen,NWTMOL,iout)
        endif
c
c========================== DDM End ====================================
c
        call isoropia(wi,rhi,tempi,cntrl,wt,gasis,aerliq,aersld,
     &                scasi,other)
c
c========================= DDM Begin ===================================
c
        if (lddm) then
          call DDMISRPIA (fca0,convfac,wtmol,sddm,nfam,nsen,NWTMOL,ierr,
     &                                                             iout)
          if (ierr.lt.0) then
            write(iout,'(a,4i4)') ' igrd,i,j,k = ',
     &                                          igrdchm,ichm,jchm,kchm
            write(iout,*) ' RH,T = ',rhi,tempi
            write(iout,*) ' CONVFAC = ',convfac
            write(iout,*) ' WI = ',(wt_save(is),is=1,5)
            write(iout,*) ' fca0 ',fca0,(sddm(ip,jdpFCRS),ip=1,nfam)
            do ip = 1, nfam
              if (fca0.ge.0.0) then
                r_1 = (     sddm(ip,jdpHNO3)*convfac
     &                +     sddm(ip,jdpPNO3)/wtmol(3)
     &                -fca0*sddm(ip,jdpFCRS)/wtmol(7))*1.e-6
              else
                r_1 = 0.0
              endif
              write(iout,*) ' SI(',ip,') = ',
     &                                sddm(ip,jdpNA  )/wtmol(5) *1.e-6,
     &      (sddm(ip,jdpSULF)*convfac+sddm(ip,jdpPSO4)/wtmol(1))*1.e-6,
     &      (sddm(ip,jdpNH3 )*convfac+sddm(ip,jdpPNH4)/wtmol(2))*1.e-6,
     &                                                             r_1,
     &      (sddm(ip,jdpHCL )*convfac+sddm(ip,jdpPCL )/wtmol(4))*1.e-6
            enddo
            call camxerr()
          endif
        endif
c
c========================== DDM End ====================================
c 
c-----Load results back to local CON array (ppm for gas, ug/m3 for aerosol)
c 
        con(ksulf) = bdnl(ksulf)                           ! sulfuric acid gas
        con(kpso4) = wt_save(2) - DBLE(con(ksulf)*1.e-6*convfac) ! sulfate
        con(kpso4) = amax1(con(kpso4)*wtmol(1)*1.e6,bdnl(kpso4))
c
        if (fca0.ge.0.0) then
          con(khno3) = sngl(gasis(2))*1.e6/convfac                 ! nitric acid gas
          con(khno3) = amax1(con(khno3),bdnl(khno3))    
          con(kpno3) = wt_save(4) - DBLE(con(khno3)*1.e-6*convfac) ! nitrate
          con(kpno3) = con(kpno3) + dno3                     ! add mineral dust nitrate
          con(kpno3) = amax1(con(kpno3)*wtmol(3)*1.e6,bdnl(kpno3))
        endif
c
        con(knh3 ) = sngl(gasis(1))*1.e6/convfac                 ! ammonia gas
        con(knh3 ) = amax1(con(knh3),bdnl(knh3))    
        con(kpnh4) = wt_save(3) - DBLE(con(knh3)*1.e-6*convfac)  ! ammonium
        con(kpnh4) = amax1(con(kpnh4)*wtmol(2)*1.e6,bdnl(kpnh4))
c
        if (khcl.ne.nspec+1) then
          con(khcl) = sngl(gasis(3))*1.e6/convfac                ! HCl gas
          con(khcl) = amax1(con(khcl),bdnl(khcl))
        endif
        if (kna.ne.nspec+1) then
          con(kna)  = amax1(con(kna),bdnl(kna))            ! sodium aerosol
          con(kpcl) = wt_save(5) - DBLE(con(khcl)*1.e-6*convfac) ! chloride
          con(kpcl) = amax1(con(kpcl)*wtmol(4)*1.e6,bdnl(kpcl))
        endif
c
        con(kph2o) = sngl(aerliq(8))                            ! aerosol water
        con(kph2o) = amax1(con(kph2o)*18.*1.e6,bdnl(kph2o))
c
c========================= Process Analysis Begin ======================
c
        if ( ldoipr ) then
          cipr(IPR_IAERO,ipa_cel,ksulf) = cipr(IPR_IAERO,ipa_cel,ksulf)
     &                                  + (con(ksulf)-csulf)*convfac
          cipr(IPR_IAERO,ipa_cel,khno3) = cipr(IPR_IAERO,ipa_cel,khno3)
     &                                  + (con(khno3)-chno3)*convfac
          cipr(IPR_IAERO,ipa_cel,knh3 ) = cipr(IPR_IAERO,ipa_cel,knh3 )
     &                                  + (con(knh3 )-cnh3 )*convfac
          cipr(IPR_IAERO,ipa_cel,kpso4) = cipr(IPR_IAERO,ipa_cel,kpso4)
     &                                  + con(kpso4)-cpso4
          cipr(IPR_IAERO,ipa_cel,kpno3) = cipr(IPR_IAERO,ipa_cel,kpno3)
     &                                  + con(kpno3)-cpno3
          cipr(IPR_IAERO,ipa_cel,kpnh4) = cipr(IPR_IAERO,ipa_cel,kpnh4)
     &                                  + con(kpnh4)-cpnh4
          if (kna.ne.nspec+1) then
            cipr(IPR_IAERO,ipa_cel,khcl) = cipr(IPR_IAERO,ipa_cel,khcl)
     &                                 + (con(khcl)-cold(khcl))*convfac
            cipr(IPR_IAERO,ipa_cel,kpcl) = cipr(IPR_IAERO,ipa_cel,kpcl)
     &                                   + con(kpcl)-cold(kpcl)
          endif
          cipr(IPR_IAERO,ipa_cel,kph2o) = cipr(IPR_IAERO,ipa_cel,kph2o)
     &                                  + con(kph2o)-cold(kph2o)
        endif
c
c========================== Process Analysis End =======================
c
      endif ! call ISORROPIA?
c
      return
c
      end
