      subroutine khetero(tcell,pcell,H2O,con,lddm,nfam,sddm)
      use chmstry
      use filunit
      implicit none
c
c----CAMx v6.30 160408
c
c     KHETERO estimates the rate constants for heterogeneous reactions of N2O5:
c       N2O5 + H2O -> 2 HNO3
c       N2O5 + HCL -> CLN2 + HNO3
c     KHETERO also sets the rate constant for hydrolysis of organic nitrate:
c       NTR2       -> HNO3
c
c     Parameterization of N2O5 on aqueous particles:
c     Bertram and Thornton (2009) Atmos. Chem. Phys., 9, 8351-8363.
c
c     Mean molecular speed formula - Seinfeld and Pandis (1998)
c     Atmospheric Chemistry and Physics, p453.
c
c     Organic nitrate hydrolysis rate:
c     Liu et al. (2012) Aerosol Sci. Technol., 46, 1359-1369.
c
c     Organic nitrate partitioning ratio:
c     Rollins et al. (2013) J. Geophys. Res., 118, 6651-6662.
c
c     Copyright 1996 - 2016
c     Ramboll ENVIRON
c
c     Modifications:
c        01/30/2006  - bkoo -     Original development
c        07/04/2007  - bkoo -     Modified to use pointer to N2O5 hydrolysis
c                                 Extended for CMU_AERO
c        03/29/2011  -cemery-     Revised to allow for inert PM with gas-phase
c                                 chemistry
c        03/28/2012  -cemery-     Revised to set rate of 0.25/hr in the
c                                 absence of PM chemistry
c        09/21/2013  - bkoo -     Revised for DDM-PM
c        10/14/2013  - bkoo -     Added hydrolysis of organic nitrate
c        01/03/2014  - bkoo -     Updated DDM-PM for organic nitrate hydrolysis
c        03/18/2014  - bkoo -     Updated for benzene SOA
c        12/07/2014  - bkoo -     Modified for VBS
c                                 Cap khetN2O5 sens
c        01/08/2016  - bkoo -     Updated for Revised VBS
c        02/29/2016  - bkoo -     Updated heterogeneous rxn of N2O5
c
c     Input arguments:
c        tcell          temperature (K)
c        pcell          pressure (mb)
c        H2O            water vapor concentration (ppm)
c        con            species concentrations (ppm or ug/m3)
c        lddm           logical flag for DDM sensitivities
c        nfam           number of sensitivity families
c        sddm           sensitivity matrix (ppm; ug/m3; 1/hr or 1/ppm-hr)
c                          sddm(:,1) [1/ppm-hr]
c                          sddm(:,2) [1/ppm-hr]
c                          sddm(:,3) [1/hr]
c
c     Output arguments:
c        sddm
c
c     Routines called:
c        NONE
c
c     Called by:
c        CHEMDRIV
c
      include 'camx.prm'
      include 'camx_aero.inc'
c
c-----Arguments
c
      real  tcell, pcell, H2O
      real  con(*)
c
c-----Local variables:
c
      real, parameter :: khon0  = 0.16667       ! [1/hr]; 4/day (particle-phase ON hydrolysis)
      real, parameter :: alfa1  = 0.34,         ! partioning parameters for total ON
     &                   alfa2  = 0.66,
     &                   cstar1 = 0.73,
     &                   cstar2 = 1000.
      real, parameter :: fntr2  = 0.71          ! NTR2 / total ON

      real :: khon                              ! psuedo gas-phase ON hydrolysis rate constant [1/hr]
      real :: fp                                ! fraction of NTR2 in the particle phase
      real :: ctoa                              ! total OA concentration [ug/m3]

      real :: qwatr, ev, es, rh                 ! relative humidity calculation

      real, parameter :: Rgc    = 8.314         ! gas constant [J/mol-K]
      real, parameter :: Pi     = 3.1415927     ! Pi
      real, parameter :: mwN2O5 = 0.108         ! MW of N2O5 [kg/mol]
      real, parameter :: rtau   = 1.66667e-3    ! reciprocal of minimum lifetime of N2O5 (10 min = 600 sec) [1/s]

      real, parameter :: Afac   = 3.2e-8        ! empirical pre-factor for gamma [s]
      real, parameter :: beta   = 1.15e6        ! parameter for k2f' [1/s]
      real, parameter :: delta  = 1.3e-1        ! parameter for k2f' [1/M]
      real, parameter :: k3k2b  = 6.0e-2        ! k3/k2b
      real, parameter :: k4k2b  = 29.0          ! k4/k2b
      real, parameter :: molarPH2O = 55.        ! molar concentration of aerosol water [M]; assume diluted aqueous phase

      real, parameter :: h2s = 3600.0           ! hour to second

      real :: vt                                ! total dry volume of fine particles
      real :: diam                              ! diameter of a fine particle [um]
      real :: nden                              ! number density of fine particles [#/cm3_air]
      real :: saden                             ! surface area density [um2/cm3_air]
      real :: cbarN2O5                          ! mean molecular speed of N2O5 [m/s]
      real :: k2fprime                          ! k2f' = k2f * [H2O] [1/s]

      real :: cak_const, r1, r2, rdenom, khet1, khet2

      integer :: l, isempty
      integer :: kwtr, isec, iaero
c
c======================== DDM Begin =======================
c
      logical :: lddm
      integer :: nfam
      integer :: ip
      real :: sddm(nfam,MXSPEC+3)
      real :: stoa,sfp
      real :: const0,sad0
      real :: dvt,dsad,dr1r
c
c======================== DDM End =======================
c
c-----No PM chemistry:
c       N2O5 + H2O -> 2 HNO3        : reset to the IUPAC 2006 rate
c       N2O5 + HCL -> CLN2 + HNO3   : use the rate set in the CHEMPARAM input file
c       NTR2       -> HNO3          : use the rate set in the CHEMPARAM input file
c
      if (aeropt.eq.'NONE' .or. aeropt.eq.'INERT') then
        rk(ihydrxn) = 2.5e-22 * 8.87e16                     ! [cm3/molecule-sec] -> [1/ppm-hr]
        return
      endif
c
c-----Hydrolysis of organic nitrate
c
      if (ihonrxn.gt.0) then
        if (aeropt.eq.'CF') then
          ctoa = con(kpoa)  + con(ksoa1) + con(ksoa2) + con(ksoa3)
     &                      + con(ksoa4) + con(ksoa5) + con(ksoa6)
     &                      + con(ksoa7) + con(ksoah) + con(ksopa)
     &                      + con(ksopb)
        elseif (aeropt.eq.'CF_VBS') then
          ctoa = con(kpas0)+con(kpas1)+con(kpas2)+con(kpas3)+con(kpas4)
     &          +con(kpbs0)+con(kpbs1)+con(kpbs2)+con(kpbs3)+con(kpbs4)
     &          +con(kpap0)+con(kpap1)+con(kpap2)+con(kpap3)+con(kpap4)
     &          +con(kpcp0)+con(kpcp1)+con(kpcp2)+con(kpcp3)+con(kpcp4)
     &          +con(kpfp0)+con(kpfp1)+con(kpfp2)+con(kpfp3)+con(kpfp4)
        elseif (aeropt.eq.'CMU') then
          ctoa = 0.0
          do isec = 1, nbin
            ctoa = ctoa + con(kpoa_1+isec-1)
     &                  + con(ksoa1_1+isec-1) + con(ksoa2_1+isec-1)
     &                  + con(ksoa3_1+isec-1) + con(ksoa4_1+isec-1)
     &                  + con(ksoa5_1+isec-1) + con(ksoa6_1+isec-1)
     &                  + con(ksoa7_1+isec-1) + con(ksoah_1+isec-1)
     &                  + con(ksopa_1+isec-1) + con(ksopb_1+isec-1)
          enddo
        else
          write(iout,'(//,a)') 'ERROR in KHETERO: invalid AEROPT'
          call camxerr()
        endif
        fp = 0.0
        if (ctoa.ge.0.0001) then
          fp = alfa1 / (1. + cstar1/ctoa) + alfa2 / (1. + cstar2/ctoa)
          fp = fp / fntr2
        endif

        qwatr = 1.e-6*H2O*18./28.8
        ev = qwatr*pcell/(qwatr + eps)
        es = e0*exp((lv/rv)*(1./273. - 1./tcell))
        rh = amin1(1.,ev/es)

        if (rh.lt.0.2) then
          rk(ihonrxn) = 0.0
        elseif (rh.ge.0.4) then
          rk(ihonrxn) = fp * khon0
        else
          rk(ihonrxn) = fp * khon0 * (rh - 0.2) / 0.2
        endif
c
c======================== DDM Begin =======================
c
        if ( lddm .and. aeropt.eq.'CF' ) then
          do ip = 1, nfam
            stoa = sddm(ip,kpoa)
     &           + sddm(ip,ksoa1) + sddm(ip,ksoa2) + sddm(ip,ksoa3)
     &           + sddm(ip,ksoa4) + sddm(ip,ksoa5) + sddm(ip,ksoa6)
     &           + sddm(ip,ksoa7) + sddm(ip,ksoah) + sddm(ip,ksopa)
     &           + sddm(ip,ksopb)
            sfp = 0.0
            if (ctoa.ge.0.0001) then
              sfp = ( fp - ( 
     &                alfa1 / ((1. + cstar1/ctoa)*(1. + cstar1/ctoa))
     &              + alfa2 / ((1. + cstar2/ctoa)*(1. + cstar2/ctoa))
     &                      ) / fntr2 ) * stoa / ctoa
            endif
            if (rh.lt.0.2) then
              sddm(ip,nspec+3) = 0.0
            elseif (rh.ge.0.4) then
              sddm(ip,nspec+3) = sfp * khon0
            else
              sddm(ip,nspec+3) = sfp * khon0 * (rh - 0.2) / 0.2
            endif
          enddo
        endif
c
c======================== DDM End =======================
c
      endif
c
c-----Heterogeneous rxn of N2O5
c
      khet1 = 0.0
      khet2 = 0.0
      cbarN2O5 = sqrt( 8.0*Rgc*tcell / (Pi*mwN2O5) )        ! [m/s]
      k2fprime = beta * ( 1. - exp(-delta*molarPH2O) )      ! [1/s]
      cak_const = cbarN2O5 * Afac * k2fprime
c
c-----CF PM chemistry
c
      if (aeropt.eq.'CF' .OR. aeropt.eq.'CF_VBS') then
        if (con(kph2o).le.bdnl(kph2o)) goto 800

        isempty = 1
        vt = 0.
        do l = ngas+1,nspec
          if (dcut(l,2).lt.(dcut(kph2o,2)+1.e-5)) then      ! fine
            if (l.ne.kph2o) then                            ! dry
              if (con(l).gt.bdnl(l)) isempty = 0
              vt = vt + con(l)/roprt(l)                     ! [ug/m3_air]/[g/m3]
            endif
          endif
        enddo
        if (isempty.eq.1) goto 800

        diam = sqrt(dcut(kph2o,1)*dcut(kph2o,2))            ! geometric mean dry diameter
        nden = vt*1.E6 / (diam*diam*diam*Pi/6.0)            ! [#/cm3_air]
c
c======================== DDM Begin =======================
c
        if ( lddm ) then
          const0 = 1.5 * cak_const / diam
          sad0 = vt**1.5 + (vt**0.5)*(con(kph2o)/roprt(kph2o))
        endif
c
c======================== DDM End =======================
c
        diam = diam * ( 1.0 +
     &         con(kph2o)/roprt(kph2o)/vt )**0.33333        ! wet diameter [um]
        saden = Pi*diam*diam*nden                           ! [um2/cm3_air]

        r1 = k3k2b * (con(kph2o)/18.)/(con(kpno3)/62.)
        r2 = k4k2b * (con(kpcl)/35.)/(con(kpno3)/62.)
        rdenom = r1 + 1. + r2
        khet1 = ( r1 / rdenom ) * cak_const * saden * 1.e-6 / 4.
        khet2 = ( r2 / rdenom ) * cak_const * saden * 1.e-6 / 4.
c
c======================== DDM Begin =======================
c
        if ( lddm ) then
          do ip = 1, nfam
            dvt = 0.0
            do l = ngas+1,nspec
              if (dcut(l,2).lt.(dcut(kph2o,2)+1.e-5)) then  ! fine
                if (l.ne.kph2o) then                        ! dry
                  dvt = dvt + sddm(ip,l)/roprt(l)
                endif
              endif
            enddo
            dsad = (vt**0.5)*( 1.5*dvt
     &                       + 0.5*dvt*con(kph2o)/roprt(kph2o)/vt
     &                       + sddm(ip,kph2o)/roprt(kph2o) )
            dsad = 0.66667 * (sad0**-0.33333) * dsad
            dr1r = ( k3k2b * (sddm(ip,kph2o)/18.) +
     &                       (sddm(ip,kpno3)/62.) +
     &               k4k2b * (sddm(ip,kpcl)/35.) ) / (con(kpno3)/62.)
            sddm(ip,nspec+1) = const0 * ( dsad * r1 / rdenom +
     &                         (sad0**0.66667) * ( k3k2b *
     &                         (sddm(ip,kph2o)/18.)/(con(kpno3)/62.) *
     &                          rdenom - r1 * dr1r ) / (rdenom**2.) )   ! [1/s]
            sddm(ip,nspec+2) = const0 * ( dsad * r2 / rdenom +
     &                         (sad0**0.66667) * ( k4k2b *
     &                         (sddm(ip,kpcl)/35.)/(con(kpno3)/62.) *
     &                          rdenom - r2 * dr1r ) / (rdenom**2.) )   ! [1/s]
          enddo
          if ( khet1.ge.rtau ) then
            sddm(:,nspec+1) = 0.0
          else
            sddm(:,nspec+1) = sddm(:,nspec+1) * h2s / H2O               ! [1/ppm-hr]
          endif
          if (ihydrxn2.gt.0) then
            sddm(:,nspec+2) = sddm(:,nspec+2) * h2s / con(khcl) -
     &                     khet2 * h2s * sddm(:,khcl) / (con(khcl)**2.) ! [1/ppm-hr]
          else
            sddm(:,nspec+2) = 0.0
          endif
        endif
c
c======================== DDM End =======================
c
c-----CMU PM chemistry
c
      elseif (aeropt.eq.'CMU') then
        kwtr = (kph2o_1 - ngas)/nbin + 1
        if (nbin.eq.1) kwtr = kph2o_1 - ngas
        do isec = 1, nbin
          if (con(kph2o_1+isec-1).le.bdnl(kph2o_1+isec-1)) cycle
          isempty = 1
          vt = 0.
          do iaero = 1, naero
            if (iaero.ne.kwtr) then
              l = ngas + (iaero - 1)*nbin + isec
              if (con(l).gt.bdnl(l)) isempty = 0
              vt = vt + con(l)/roprt(l)
            endif
          enddo
          if (isempty.eq.1) cycle

          diam = sqrt(dcut(ngas+isec,1)*dcut(ngas+isec,2))
          nden = vt*1.E6 / (diam*diam*diam*Pi/6.0)
          diam = diam * ( 1.0 +
     &           con(kph2o_1+isec-1)/roprt(kph2o_1+isec-1)/vt )**0.33333
          saden = Pi*diam*diam*nden

          r1 = k3k2b * (con(kph2o_1+isec-1)/18.)
     &               / (con(kpno3_1+isec-1)/62.)
          r2 = k4k2b * (con(kpcl_1+isec-1)/35.)
     &               / (con(kpno3_1+isec-1)/62.)
          rdenom = r1 + 1. + r2
          khet1 = khet1 +
     &          ( r1 / rdenom ) * cak_const * saden * 1.e-6 / 4.
          khet2 = khet2 +
     &          ( r2 / rdenom ) * cak_const * saden * 1.e-6 / 4.
        enddo
      else
        write(iout,'(//,a)') 'ERROR in KHETERO: invalid AEROPT'
        call camxerr()
      endif
c
c --- adjust rk(N2O5); rk(N2O5) will be reset by KTHERM every timestep
c
 800  khet1 = MIN( khet1, rtau )                            ! check against upper limit [1/s]
      rk(ihydrxn) = rk(ihydrxn) + khet1 * h2s / H2O         ! add gas-phase rate; [1/ppm-hr]
      if (ihydrxn2.gt.0) rk(ihydrxn2) = khet2 * h2s / con(khcl)

      return

      end
