      subroutine wetdeprt(m1,m2,m3,ixcl,jycl,nlay,nrtsp,deltat,area,
     &                    depth,tempk,press,cwc,pwr,pws,pwg,cph,densfac,
     &                    conc,solmass,vegmass,fsurf)
      use chmstry
      use rtracchm
c
c----CAMx v6.30 160408
c 
c     This version is for the RTRAC species.
c     WETDEP modifies vertical concentration profiles for a given grid via 
c     precipitation processes.  This subroutine has been completely rewritten
c     for CAMx v4.
c 
c     Copyright 1996 - 2016
c     Ramboll ENVIRON
c           
c     Modifications:
c        05/01/03   Taken from WETDEP for regular model
c        06/15/03   Fixed bug in scaling of tmass
c        02/11/04   Updated empirical rainfall relationships
c        05/03/05   Revised gas scavenging calculation
c        06/20/05   Revised to handle liquid/frozen cloud and precip water
c        10/21/05   Added dissociation to Henry's Law for NH3, HNO3, SO2
c        08/31/06   Added map scale factor
c        10/29/09   Added flux of RTRAC species to surface model
c        12/22/09   Improved treatment of "c0" array
c        10/09/14   Restructured for subgrid convective model
c        03/17/16   Fixed bug to dissallow tmass<0 (Ondrej Vlcek, CHMI)
c
c     Input arguments:
c        m1                  number of slice columns
c        m2                  number of slice rows
c        m3                  number of slice layers
c        ixcl                i-cell index
c        jycl                j-cell index
c        nlay                number of layers 
c        nrtsp               number of species in RTRAC array
c        deltat              time step (s)
c        area                cell area (m2)
c        depth               cell depth (m)
c        tempk               temperature field (K)
c        press               pressure field (mb)
c        cwc                 cloud water content (g/m3)
c        pwr                 rain water content (g/m3)
c        pws                 snow water content (g/m3)
c        pwg                 graupel water content (g/m3)
c        cph                 cloud water pH
c        densfac             factor to convert to umol/m3
c        conc                concentration field (umol/m3, ug/m3)
c        solmass             surface soil mass field (mol/ha, g/ha)
c        vegmass             surface veg mass field (mol/ha, g/ha)
c        fsurf               fractional landuse coverage
c             
c     Output arguments: 
c        conc                concentration field (umol/m3, ug/m3)
c        solmass             surface soil mass field (mol/ha, g/ha)
c        vegmass             surface veg mass field (mol/ha, g/ha)
c             
c     Routines called: 
c        SCAVRAT
c        HENRYFNC
c             
c     Called by: 
c        CIGDRIVE
c 
      implicit none
      include 'camx.prm'
      include 'deposit.inc'
      include 'rtracsrf.inc'
c
      integer m1,m2,m3,ixcl,jycl,nlay,nrtsp
      real deltat,area,densfac
      real, dimension(nlay) :: tempk,press,cwc,pwr,pws,pwg,cph,depth
      real, dimension(MXLAYER,MXSPEC) :: conc
      real, dimension(m1,m2,nrtsp) :: solmass,vegmass
      real, dimension(m1,m2,NLUW89) :: fsurf

      logical lcloud,ltop,lgraupl
      integer kbot,ktop,k,ncnt,l,n
      integer iwater
      real rd,rhoh2o,cellvol,rainvol,rhoair,delc,delm,convfac,cmin,hlaw,
     &     gscav,ascav,totc,totw,ceq,psize,cwat,pwat,rconst,dtfall,
     &     drpvel,delc0,depmas
c
      real pp(MXLAYER)
      real rr(MXLAYER)
      real volrat(MXLAYER)
      real c0(MXTRSP)
      real tmass(MXTRSP)
c
      data rd /287./         ! Dry air gas constant (J/K/kg)
      data rhoh2o /1.e6/     ! water density (g/m3)
      data rconst /8.206e-2/ ! gas constant (l.atm/mol.K)
c
c-----Entry point
c
      do l = 1,nrtsp
        tmass(l) = 0.
        c0(l) = 0.
      enddo
c
c-----Scan column for layers containing precipitation bottom/top
c
      kbot = 0
      ktop = 0
      do k = 1,nlay
        if (pwr(k).ge.cwmin .or. pws(k).ge.cwmin .or.
     &      pwg(k).ge.cwmin) then
          kbot = k
          goto 25
        endif
      enddo
      goto 20
c
  25  continue
      if (kbot.eq.nlay) goto 20
      ncnt = 1
      do k = kbot+1,nlay
        if (pwr(k).lt.cwmin .and. pws(k).lt.cwmin .and.
     &      pwg(k).lt.cwmin) then
          ktop = k-1
          goto 26
        endif
        ncnt = ncnt + 1
      enddo
      ktop = nlay
  26  continue
      if (kbot.gt.1 .and. ncnt.eq.1) goto 20
c
c-----Load precip and cloud profiles, and determine precip rate
c
      do k = 1,nlay
        volrat(k) = 0.
        rr(k) = 0.
        pp(k) = pwr(k) + pws(k) + pwg(k)
      enddo
      do k = kbot,ktop
        volrat(k) = pp(k)/rhoh2o                 ! drop volume/air volume
        rr(k) = (volrat(k)/1.0e-7)**1.27         ! rainfall rate (mm/hr)
      enddo
c
c-----Loop over layers and species for precipitating columns
c
      ltop = .true.
      do 30 k = ktop,kbot,-1
        lcloud = .false.
        lgraupl = .false.
        if (cwc(k).ge.cwmin) lcloud = .true.
        if (pwg(k).ge.cwmin) lgraupl = .true.
        cellvol = area*depth(k)
        rainvol = volrat(k)*cellvol
        rhoair = 100.*press(k)/(rd*tempk(k))
c
c-----Calculate scavenging for soluble gas species
c
        do 40 l = 1,nrtgas
          if( rthlaw(l).LT.1.e-6 ) goto 40
          call henryfnc(0,rthlaw(l),rttfact(l),tempk(k),cph(k),knh3,
     &                  khno3,kso2,hlaw)
          hlaw = hlaw*rconst*tempk(k)
          cwat = cwc(k)
          pwat = pp(k)
          if (tempk(k).lt.273. .and. rtscale(l).gt.0.) then
            cwat = amax1(0.,cwc(k)*(tempk(k) - tamin)/(273. - tamin))
            pwat = pwr(k)
          endif
          delc = 0.
          delm = 0.
          convfac = densfac*(273./tempk(k))*(press(k)/1013.)
          cmin = rtlbnd(l)*convfac
          conc(k,l) = amax1(cmin,conc(k,l))
c
          call scavrat(.false.,lcloud,lgraupl,tamin,rr(k),tempk(k),cwat,
     &                 rhoair,conc(k,l),hlaw,rtdrate(l),rtscale(l),0.,
     &                 0.,gscav,ascav,drpvel)
c
          totc = conc(k,l) + c0(l)
          totw = cwat + pwat
          ceq = totc/(1. + hlaw*totw/rhoh2o)
          ceq = totc - ceq
          if( totw .NE. 0.0 ) then
             ceq = ceq*pwat/totw
          else
             ceq = 0.0
          endif
          delc = (ceq - c0(l))*(1. - exp(-gscav*deltat))
          if (delc.gt.0.) delc = amin1(delc,conc(k,l)-cmin)
          dtfall = depth(k)/drpvel
          delc0 = (ceq - c0(l))*(1. - exp(-gscav*dtfall))
          if (delc0.gt.0.) delc0 = amin1(delc0,conc(k,l)-cmin)
          delm = delc*cellvol
          if (delm.lt.0.) then
            delm = max(delm,-tmass(l))
            delc = delm/cellvol
          endif
c
c-----Update the cell concentration and rain mass
c
          conc(k,l) = conc(k,l) - delc
          c0(l) = max(c0(l)+delc0,0.)
          tmass(l) = max(tmass(l)+delm,0.)
 40     continue
c
c-----Calculate scavenging for particulate species
c
        if (nrtaero .gt. 0) then
          do 50 l = nrtgas+1,nrtrac
            delc = 0.
            delm = 0.
            cmin = rtlbnd(l)
            conc(k,l) = amax1(cmin,conc(k,l))
            psize = 1.e-6*sqrt(rtlcut(l)*rtucut(l))

            call scavrat(.TRUE.,lcloud,lgraupl,tamin,rr(k),
     &                   tempk(k),0.,rhoair,0.,0.,0.,0.,
     &                   psize,rtdens(l),gscav,ascav,drpvel)

            delc = conc(k,l)*(1. - exp(-ascav*deltat))
            delc = amin1(delc,conc(k,l)-cmin)
c
            conc(k,l) = conc(k,l) - delc
            delm = delc*cellvol
            tmass(l) = tmass(l) + delm
 50       continue
        endif
        ltop = .false.
 30   continue
c
c-----If rain evaporates before reaching the ground, return all mass
c     back to layer KBOT
c
      if (kbot.gt.1) then
        cellvol = area*depth(kbot)
        do l = 1,nrtrac
          conc(kbot,l) = conc(kbot,l) + tmass(l)/cellvol
          tmass(l) = 0.
        enddo
c
c-----Otherwise rain reaches the ground, increment surface mass array
c
      elseif (lsrfmodrt) then
        do l = 1,nrtrac
          depmas = 1.e-2*tmass(l)/area
          do n = 1,NLUW89
            iwater = 7
            if (n.ne.iwater) then
              solmass(ixcl,jycl,l) = solmass(ixcl,jycl,l) +
     &                                fsoil(n)*fsurf(ixcl,jycl,n)*depmas
              vegmass(ixcl,jycl,l) = vegmass(ixcl,jycl,l) +
     &                         (1. - fsoil(n))*fsurf(ixcl,jycl,n)*depmas
            endif
          enddo
        enddo
      endif
c
 20   return
      end
