      subroutine soachem(iout,noxin,nprin,nCG,tempk,press,dthr,
     &                   avgox,cprec,cCG,cCGH,
     &                   nfam,avgsen,sen_prec,sen_CG,sen_CGH,lddm)
c
c----CAMx v6.30 160408
c
c-----------------------------------------------------------------------
c   Description:
c     This routine solves the oxidation reactions for SOA precursors
c     assuming first-order decay and constant oxidant concentrations
c
c     Anthropogenic:
c
c        TOLA + OH -> TRO2                                ; 1.80e-12 @ -340.
c                     TRO2 + NO  -> 0.036 CG1 + 0.069 CG2 ; 2.70e-12 @ -360.
c                     TRO2 + HO2 -> 0.22  CGH             ; 1.90e-13 @ -1300.
c        XYLA + OH -> XRO2                                ; 1.85e-11
c                     XRO2 + NO  -> 0.022 CG1 + 0.064 CG2 ; 2.70e-12 @ -360.
c                     XRO2 + HO2 -> 0.21  CGH             ; 1.90e-13 @ -1300.
c        BNZA + OH -> BRO2                                ; 2.30e-12 @  190.
c                     BRO2 + NO  -> 0.037 CG1 + 0.46  CG2 ; 2.70e-12 @ -360.
c                     BRO2 + HO2 -> 0.19  CGH             ; 1.90e-13 @ -1300.
c
c     Biogenic:
c
c        ISP  + OH  -> 0.015 CG3 + 0.12 CG4  ; 2.70e-11 @ -390.
c        ISP  + O3  ->                       ; 1.03e-14 @ 1995.
c        ISP  + NO3 ->                       ; 3.03e-12 @ 448.
c
c        TRP  + O   -> 0.065 CG5 + 0.29 CG6  ; 3.60e-11
c        TRP  + OH  -> 0.065 CG5 + 0.29 CG6  ; 1.50e-11 @ -449.
c        TRP  + O3  -> 0.065 CG5 + 0.29 CG6  ; 1.20e-15 @ 821.
c        TRP  + NO3 -> 0.065 CG5 + 0.29 CG6  ; 3.70e-12 @ -175.
c       (a-pinene)
c
c        SQT  + OH  -> 0.85 CG7              ; 1.97e-10
c        SQT  + O3  -> 0.85 CG7              ; 1.16e-14
c        SQT  + NO3 -> 0.85 CG7              ; 1.90e-11
c       (b-caryophellene)
c
c     Rate constant:
c        k = A exp(-E/T)                     ; A [cm3/molecule-sec] @ E [K]
c
c     Copyright 1996 - 2016
c     Ramboll ENVIRON
c
c     Input arguments:
c        iout     standard output file unit
c        noxin    number of oxidants
c        nprin    number of precursors
c        nCG      number of CG species
c        tempk    cell temperature [K]
c        press    cell pressure [mb]
c        dthr     time duration to be integrated [hr]
c        avgox    average oxidant concentrations [ppm]
c                 1 - O; 2 - OH; 3 - O3; 4 - NO3; 5 - NO; 6 - HO2
c        cprec    precursor concentrations [ppm]
c                 1 - TOLA; 2 - XYLA; 3 - BNZA; 4 - ISP; 5 - TRP; 6 - SQT
c        cCG      CG species concentrations [ppm]
c                 1 - CG1; 2 - CG2; ... ; 7 - CG7
c        nfam     number of sensitivity families
c        avgsen   average sensitivities of oxidants [ppm]
c        sen_prec sensitivities of precursors [ppm]
c        sen_CG   sensitivities of CGs [ppm]
c        lddm     logical flag for DDM sensitivities
c
c     Output arguments:
c        cprec    precursor concentrations [ppm]
c        cCG      CG species concentrations [ppm]
c        cCGH     CGH species concentration [ppm]
c        sen_prec sensitivities of precursors [ppm]
c        sen_CG   sensitivities of CGs [ppm]
c        sen_CGH  sensitivities of CGH [ppm]
c
c-----------------------------------------------------------------------
c   LOG:
c-----------------------------------------------------------------------
c
c     03/22/06  --bkoo--  Original development
c     06/20/06  --bkoo--  2-product model for isoprene (Henze & Seinfeld, 2006)
c     08/23/13  --bkoo--  Added code for DDM-PM
c     03/18/14  --bkoo--  Revised for benzene SOA
c
c-----------------------------------------------------------------------
c
      include 'soap.inc'

      integer     noxin,nprin,nCG
      real        tempk,press,dthr,avgox(noxin),cprec(nprin),cCG(nCG)
      real        cCGH

      integer     nrxn, nox, nprec, naro
      parameter ( nrxn  = 15 )           ! # of reactions
      parameter ( nox   = 6  )
      parameter ( nprec = 6  )
      parameter ( naro  = 3  )           ! # of aromatics
c
c======================== DDM Begin =======================
c
      integer     nfam,ifam
      real        avgsen(nfam,noxin)
      real        sen_prec(nfam,nprin)
      real        sen_CG(nfam,nCG)
      real        sen_CGH(nfam)
      real        srhi(nfam)
      real        ssumk,srk1(nox),sdprec,sdfac
      logical     lddm
c
c======================== DDM End =======================
c
      real*8      rkpar(2,nrxn) ! rate constant parameters
c
      real        yval(2,nrxn) ! yield [ppm/ppm]
      real        yvalh(naro)  ! low-NOx yield [ppm/ppm]
      integer     idCG(2,nrxn) ! CG ID for the oxidation products
      integer     mox(nprec),idox(nox,nprec),idrxn(nox,nprec)
      real*8      cf0, cfac
c                       A         E
      data  rkpar / 1.80d-12, -340.d0,  ! rxn 1 - TOLA + OH
     &              1.85d-11,    0.d0,  ! rxn 2 - XYLA + OH
     &              2.30d-12,  190.d0,  ! rxn 3 - BNZA + OH
     &              2.70d-11, -390.d0,  ! rxn 4 - ISP + OH
     &              1.03d-14, 1995.d0,  ! rxn 5 - ISP + O3
     &              3.03d-12,  448.d0,  ! rxn 6 - ISP + NO3
     &              3.60d-11,    0.d0,  ! rxn 7 - TRP + O
     &              1.50d-11, -449.d0,  ! rxn 8 - TRP + OH
     &              1.20d-15,  821.d0,  ! rxn 9 - TRP + O3
     &              3.70d-12, -175.d0,  ! rxn 10- TRP + NO3
     &              1.97d-10,    0.d0,  ! rxn 11- SQT + OH
     &              1.16d-14,    0.d0,  ! rxn 12- SQT + O3
     &              1.90d-11,    0.d0,  ! rxn 13- SQT + NO3
     &              2.70d-12, -360.d0,  ! rxn 14- ?RO2 + NO
     &              1.90d-13,-1300.d0 / ! rxn 15- ?RO2 + HO2

c                  Product 1    Product 2
      data  yval  / 0.036,       0.069,  ! rxn 1
     &              0.022,       0.064,  ! rxn 2
     &              0.037,       0.46 ,  ! rxn 3
     &              0.015,       0.12 ,  ! rxn 4
     &              0.0  ,       0.0  ,  ! rxn 5
     &              0.0  ,       0.0  ,  ! rxn 6
     &              0.065,       0.29 ,  ! rxn 7
     &              0.065,       0.29 ,  ! rxn 8
     &              0.065,       0.29 ,  ! rxn 9
     &              0.065,       0.29 ,  ! rxn 10
     &              0.85 ,       0.0  ,  ! rxn 11
     &              0.85 ,       0.0  ,  ! rxn 12
     &              0.85 ,       0.0  ,  ! rxn 13
     &              0.0  ,       0.0  ,  ! rxn 14
     &              0.0  ,       0.0   / ! rxn 15

c                   TOLA  XYLA  BNZA
      data  yvalh / 0.22, 0.21, 0.19 /

c                  Product 1    Product 2
      data  idCG  / 1,           2,      ! rxn 1
     &              1,           2,      ! rxn 2
     &              1,           2,      ! rxn 3
     &              3,           4,      ! rxn 4
     &              3,           4,      ! rxn 5
     &              3,           4,      ! rxn 6
     &              5,           6,      ! rxn 7
     &              5,           6,      ! rxn 8
     &              5,           6,      ! rxn 9
     &              5,           6,      ! rxn 10
     &              7,           7,      ! rxn 11
     &              7,           7,      ! rxn 12
     &              7,           7,      ! rxn 13
     &              0,           0,      ! rxn 14
     &              0,           0 /     ! rxn 15

      data  mox   / 1,  1,  1,  3,  4,  3 / ! # of oxidants to react
c                  Oxidant ID
      data  idox  / 2,  0,  0,  0,  0,  0,  ! TOLA
     &              2,  0,  0,  0,  0,  0,  ! XYLA
     &              2,  0,  0,  0,  0,  0,  ! BNZA
     &              2,  3,  4,  0,  0,  0,  ! ISP
     &              1,  2,  3,  4,  0,  0,  ! TRP
     &              2,  3,  4,  0,  0,  0 / ! SQT
c                  Reaction ID 
      data  idrxn / 1,  0,  0,  0,  0,  0,  ! TOLA
     &              2,  0,  0,  0,  0,  0,  ! XYLA
     &              3,  0,  0,  0,  0,  0,  ! BNZA
     &              4,  5,  6,  0,  0,  0,  ! ISP
     &              7,  8,  9, 10,  0,  0,  ! TRP
     &             11, 12, 13,  0,  0,  0 / ! SQT

c     [1/ppm-hr]=[cm3/molecule-sec]*3600*(6.022e23*1.e-12/8.314)*(P[Pa]/T[K])
      data  cf0 / 2.608d14 /      ! 3600*(6.022e23*1.e-12/8.314)

      real        rktmp(nrxn), rk1(nox), sumk, dprec, dfac
      real        rhi, rlo
      integer     i, j, k

      save        rkpar, yval, yvalh, idCG, mox, idox, idrxn, cf0

      logical, save :: lfirsttime = .true.
c
c-----Check consistency
c
      if (lfirsttime) then
        lfirsttime = .false.
        if (noxin.ne.nox) then
          write(iout,'(//,A)') 'ERROR in SOACHEM:'
          write(iout,'(1X,A)') 'Inconsistency in # of oxidants'
          write(iout,'(2X,A,I5)')'Input: ',noxin
          write(iout,'(2X,A,I5)')'Local: ',nox
          call camxerr()
        endif
        if (nprin.ne.nprec) then
          write(iout,'(//,A)') 'ERROR in SOACHEM:'
          write(iout,'(1X,A)') 'Inconsistency in # of precursors'
          write(iout,'(2X,A,I5)')'Input: ',nprin
          write(iout,'(2X,A,I5)')'Local: ',nprec
          call camxerr()
        endif
        if (nCG.ne.nsoap) then
          write(iout,'(//,A)') 'ERROR in SOACHEM:'
          write(iout,'(1X,A)') 'Inconsistency in # of CG/SOA pairs'
          write(iout,'(2X,A,I5)')'Input   : ',nCG
          write(iout,'(2X,A,I5)')'SOAP.INC: ',nsoap
          call camxerr()
        endif
      endif
c
c-----Conversion factor for rate constants
c
      cfac = cf0 * DBLE(press*100./tempk) ! [1/ppm-hr] = cfac * [cm3/molecule-sec]
c
c-----Rate constants in [1/ppm-hr]
c
      do i = 1, nrxn
        rktmp(i) = SNGL( rkpar(1,i)*dexp(-rkpar(2,i)/DBLE(tempk))*cfac )
      enddo
c
c-----Calculate the amount of precursor reacted / CG produced (aromatics)
c
      rhi  =       rktmp(14) * avgox(5)
      sumk = rhi + rktmp(15) * avgox(6)
      rhi  = rhi / sumk
      rlo  = 1.0 - rhi
      cCGH = 0.0
c
c======================== DDM Begin =======================
c
      if ( lddm ) then
        do ifam = 1, nfam
          srhi(ifam) = (          rktmp(14) * avgsen(ifam,5)
     &                  - rhi * ( rktmp(14) * avgsen(ifam,5)
     &                          + rktmp(15) * avgsen(ifam,6) ) ) / sumk
          sen_CGH(ifam) = 0.0
        enddo
      endif
c
c======================== DDM End =======================
c
      do j = 1, naro
        rk1(1) = rktmp(idrxn(1,j)) * avgox(idox(1,j))
        dprec = cprec(j) * ( 1. - exp(-rk1(1)*dthr) )
        if (dprec.lt.1.e-12) cycle
        cprec(j) = cprec(j) - dprec
        dfac = rhi * dprec
        cCG(idCG(1,idrxn(1,j))) = cCG(idCG(1,idrxn(1,j)))       ! 1st product
     &                              + yval(1,idrxn(1,j)) * dfac
        cCG(idCG(2,idrxn(1,j))) = cCG(idCG(2,idrxn(1,j)))       ! 2nd product
     &                              + yval(2,idrxn(1,j)) * dfac
        cCGH = cCGH + yvalh(j) * rlo * dprec
c
c======================== DDM Begin =======================
c
        if ( lddm ) then
          do ifam = 1, nfam
            srk1(1) = rktmp(idrxn(1,j)) * avgsen(ifam,idox(1,j))
            sdprec = sen_prec(ifam,j)*(1.-exp(-rk1(1)*dthr)) + 
     &                                            cprec(j)*dthr*srk1(1)
            sen_prec(ifam,j) = sen_prec(ifam,j) - sdprec
            sdfac = srhi(ifam) * dprec + rhi * sdprec
            sen_CG(ifam,idCG(1,idrxn(1,j))) =
     &                       sen_CG(ifam,idCG(1,idrxn(1,j))) +
     &                                       yval(1,idrxn(1,j)) * sdfac
            sen_CG(ifam,idCG(2,idrxn(1,j))) =
     &                       sen_CG(ifam,idCG(2,idrxn(1,j))) +
     &                                       yval(2,idrxn(1,j)) * sdfac
            sen_CGH(ifam) = sen_CGH(ifam) + yvalh(j) * 
     &                            ( rlo * sdprec - srhi(ifam) * dprec )
          enddo
        endif
c
c======================== DDM End =======================
c
      enddo
c
c-----Calculate the amount of precursor reacted / CG produced (biogenics)
c
      do j = naro+1, nprec
        sumk = 0.0
        do k = 1, mox(j)
          rk1(k) = rktmp(idrxn(k,j)) * avgox(idox(k,j))
          sumk = sumk + rk1(k)
        enddo
        dprec = cprec(j) * ( 1. - exp(-sumk*dthr) )
        if (dprec.lt.1.e-12) cycle
        cprec(j) = cprec(j) - dprec
        do k = 1, mox(j)
          dfac = rk1(k) / sumk * dprec
          cCG(idCG(1,idrxn(k,j))) = cCG(idCG(1,idrxn(k,j)))       ! 1st product
     &                                + yval(1,idrxn(k,j)) * dfac
          cCG(idCG(2,idrxn(k,j))) = cCG(idCG(2,idrxn(k,j)))       ! 2nd product
     &                                + yval(2,idrxn(k,j)) * dfac
        enddo
c
c======================== DDM Begin =======================
c
        if ( lddm ) then
          do ifam = 1, nfam
            ssumk = 0.0
            do k = 1, mox(j)
              srk1(k) = rktmp(idrxn(k,j)) * avgsen(ifam,idox(k,j))
              ssumk = ssumk + srk1(k)
            enddo
            sdprec = sen_prec(ifam,j)*(1.-exp(-sumk*dthr)) + 
     &                                            cprec(j)*dthr*ssumk
            sen_prec(ifam,j) = sen_prec(ifam,j) - sdprec
            do k = 1, mox(j)
              dfac = rk1(k) / sumk
              sdfac = (srk1(k)-dfac*ssumk)/sumk*dprec + dfac*sdprec
              sen_CG(ifam,idCG(1,idrxn(k,j))) =
     &                       sen_CG(ifam,idCG(1,idrxn(k,j))) +
     &                                       yval(1,idrxn(k,j)) * sdfac
              sen_CG(ifam,idCG(2,idrxn(k,j))) =
     &                       sen_CG(ifam,idCG(2,idrxn(k,j))) +
     &                                       yval(2,idrxn(k,j)) * sdfac
            enddo
          enddo
        endif
c
c======================== DDM End =======================
c
      enddo

      return
      end

