      subroutine soap(ntot,caer,cgas,MWCGH,soah,sopa,sopb,cpre,
     &                tempk,convfac,dt,iout,igrdchm,ichm,jchm,kchm,
     &                lppm,apoly,csatT)
      implicit none
c
c**********************************************************************
c                                                                     * 
c                                                                     * 
c  SOAP - A SUBROUTINE TO CALCULATE GAS-AEROSOL PARTITIONING OF       *
c                     SECONDARY ORGANIC SPECIES                       *
c                                                                     *
c                           DEVELOPED BY:                             *
c                                                                     * 
c                           ROSS STRADER                              *
c            DEPARTMENT OF CIVIL/ENVIRONMENTAL ENGINEERING            *
c             CARNEGIE MELLON UNIVERSITY, 5000 FORBES AVE             *
c                   PITTSBURGH, PENNSYLVANIA, 15213                   *
c                                                                     * 
c                               AND                                   *
c                                                                     * 
c                         SPYROS N. PANDIS                            *
c       DEPARTMENTS OF CHEMICAL ENGINEERING AND ENGINEERING AND       *
c                          PUBLIC POLICY                              *
c             CARNEGIE MELLON UNIVERSITY, 5000 FORBES AVE             *
c                   PITTSBURGH, PENNSYLVANIA, 15213                   *
c                                                                     * 
c                                                                     * 
c                                                                     * 
c                                                                     * 
c      ASSOCIATED SUBROUTINES:                                        *
c      CALC       - CALCULATES THE MOLE FRACTION OF EACH SOLUTION-    *
c                   FORMING COMPOUND IN THE PSEUDO-IDEAL SOLUTION     * 
c                   (AEROSOL PHASE)                                   *
c      FCN        - NONLINEAR SYSTEM TO BE SOLVED BY HYBRD            *
c      RNDNUM     - GENERATES RANDOM NUMBERS                          *
c      HYBRD      - SOLVES A SYSTEM OF NONLINEAR EQUATIONS            *
c                                                                     * 
c                                                                     * 
c**********************************************************************
c                                                                     * 
c                                                                     * 
c     THIS MODEL USES A PSEUDO-IDEAL SOLUTION THEORY TO PARTITION     *
c     SECONDARY ORGANIC SPECIES BETWEEN THE AEROSOL AND GAS PHASE.    *
c     THE THEORY IS OUTLINED IN:                                      * 
c                                                                     * 
c     STRADER, R. (1998), 'EVALUATION OF SECONDARY ORGANIC AEROSOL    *
c          FORMATION IN WINTER,' M.S. THESIS, DEPT. OF CIVIL/         *
c          ENVIRONMENTAL ENGINEERING, CARNEGIE MELLON UNIVERSITY      *
c                                                                     * 
c                                                                     * 
c**********************************************************************
c
c
c  MODIFICATIONS
c
c     Jan 2003 -gyarwood- Revised for CAMx
c     01/20/03 -bkoo-     Revised for CAMx
c     03/09/03 -bkoo-     Revised for new SOAP driver
c     05/27/05 -bkoo-     Revised for new bi-section solver
c     12/29/06 -bkoo-     Revised for the updated SOA scheme
c     04/13/10 -bkoo-     Approximate solutions if POA is dominant
c     08/23/13 -bkoo-     Revised for DDM-PM
c     03/18/14 -bkoo-     Revised for benzene SOA
c
c DEFINITION OF VARIABLES:
c
c  INPUTS
c
c     ntot    - total number of CG/SOA pairs
c     caer    - aerosol-phase concentrations of SOA species (ug/m3)
c     cgas    - gas-phase concentrations of CG species (ppm or ug/m3)
c     MWCGH   - molecular weight of CGH (for consistency check)
c     soah    - SOAH concentration (ug/m3)
c     sopa    - polymerized SOA (anthropogenic) (ug/m3)
c     sopb    - polymerized SOA (biogenic) (ug/m3)
c     cpre    - POA (pre-existing OA) concentration (ug/m3)
c     tempk   - temperature (K)
c     convfac - conversion factor: umol/m3 = ppm * convfac 
c     dt      - time step (hours)
c     iout    - standard output file unit
c     igrdchm - index for grid containing the grid cell
c     ichm    - i index of grid cell
c     jchm    - j index of grid cell
c     kchm    - k index of grid cell
c     lppm    - gases in ppm if true, ug/m3 if false
c
c   OUTPUTS
c
c     caer    - aerosol-phase concentrations of SOA species (ug/m3)
c     cgas    - gas-phase concentrations of CG species (ppm or ug/m3)
c     apoly   - polymerized mass for each SOA (ug/m3)
c     csatT   - saturation concentrations of CG/SOA pairs at current T 
c               (ug/m3)
c
c   VARIABLES USED WITHIN SOAP
c
c     i       - counter
c     icont   - counter
c     sum     - counter
c     nsol    - total number of solution-forming SOA species
c     cstemp  - temperatures corresponding to saturation concentrations
c               of CG/SOA species (K)
c     csat    - saturation concentrations of CG/SOA species (ug/m3)
c     deltah  - enthalpy of vaporization of CG/SOA species (J/mol)
c     flagsoap- set to 1 if CG/SOA species forms solutions; 0 if not
c     mwsoap  - molecular weights of CG/SOA species (g/mol)
c     scaer   - aerosol-phase concentrations of solution-forming 
c               SOA species (ug/m3)
c     scgas   - gas-phase concentrations of solution-forming 
c               SOA species (ug/m3)
c     scsat   - saturation concentrations of solution-forming 
c               SOA species (ug/m3)
c     sctot   - total concentrations of solution-forming SOA species
c               (ug/m3)
c     smw     - molecular weights of solution-forming SOA species 
c               (g/mol)
c     znum    - counter for number of iterations
c     conmin  - use simple solution for species below this level (ug/m3)
c     cpremin - no pre-existing organics if cpre < cpremin (ug/m3) 
c     xtol    - error tolerance for bi-section method
c
c***********************************************************************
c
c VARIABLE DECLARATION
c
      include 'soap.inc'
      real        conmin, cpremin, xtola, xtolr
      parameter ( conmin  = 1.e-6 )
      parameter ( cpremin = 1.01e-9 )
      parameter ( xtola   = 1.e-8 )
      parameter ( xtolr   = 1.e-6 )
c
      integer     ntot
      real        caer(ntot), cgas(ntot), ctot(NSOAP), csatT(NSOAP)
      real        smw(NSOAP), scsat(NSOAP)
      real        sctot(NSOAP), scaer(NSOAP), scgas(NSOAP)
      integer     idx(NSOAP)
c
      real        MWCGH, soah, sopa, sopb, cpre, mwpre
      real        apoly(ntot), fpoly, dt
      real        tempk, convfac, sum
      integer     iout, igrdchm, ichm, jchm, kchm
      integer     i, icont, nsol, znum
      logical     lppm

      real        cpx, bb, cc, xend, fend, xmid, fmid, dx

      logical, save :: lfirsttime = .true.
c
c***********************************************************************
c
c Entry point
c
      if (lfirsttime) then
        lfirsttime = .false.
        if (ntot.ne.NSOAP) then 
          write(iout,'(//,A)') 'ERROR in SOAP:'
          write(iout,'(1X,A)') 'Inconsistency in # of CG/SOA pairs'
          write(iout,'(2X,A,I5)') 'NSOAP (SOAP.INC) = ',NSOAP
          write(iout,'(2X,A,I5)') 'Argument to SOAP = ',ntot
          call camxerr()
        endif
        if (MWCGH.ne.mwsoah) then
          write(iout,'(//,A)') 'ERROR in SOAP:'
          write(iout,'(1X,A)') 'Inconsistency in molecular weight'
          write(iout,'(2X,A,F10.4)') 'MW(CGH)  = ',MWCGH
          write(iout,'(2X,A,F10.4)') 'MW(SOAH) = ',mwsoah
          call camxerr()
        endif
      endif

      do i=1,ntot
        if (lppm) cgas(i) = cgas(i)*convfac*mwsoap(i) ! convert to ug/m3
        ctot(i) = caer(i) + cgas(i)
      enddo
c
c Check PFLAG to set pre-existing organic mass
c
      if(PFLAG.eq.0) cpre = 0.0
      cpx = cpre/mwpoa
c
c Add non-volatile SOAs to the pre-existing organic mass
c
      cpre = cpre + soah + sopa + sopb
      cpx = cpx + soah/mwsoah + sopa/mwsopa + sopb/mwsopb
      if (cpx.eq.0.0) then
        mwpre = 1.0
      else
        mwpre = cpre / cpx
      endif
c
c CHANGE SATURATION CONCENTRATIONS ACCORDING TO CURRENT TEMPERATURE
c
      do i=1,ntot
         csatT(i)=csat(i)*(cstemp(i)/tempk)*exp((deltah(i)/8.314)
     &                *(1/cstemp(i)-1/tempk))
      enddo     
c
c CALCULATE AEROSOL-PHASE CONCENTRATION (CAER) AND GAS-PHASE 
c CONCENTRATION (CGAS) FOR NON-SOLUTION-FORMING COMPOUNDS
c COMPOUNDS THAT HAVE A CONCENTRATION OF LESS THAN conmin ARE IGN0RED
c MAP COMPOUNDS THAT FORM SOLUTIONS ONTO ARRAYS
c
      icont=0
      do i=1,ntot
         if (flagsoap(i).eq.0) then
            cgas(i) = amin1(ctot(i), csatT(i))
            caer(i) = ctot(i) - cgas(i)
c
c========================= DDM Begin ===================================
c
            CALL DDMSOATRV (ctot(i),csatT(i),0.0,0.0,0.0,i,1,iout)
c
c========================== DDM End ====================================
c
         elseif (ctot(i).lt.conmin) then
            cgas(i) = ctot(i)
            caer(i) = 0.0
c
c========================= DDM Begin ===================================
c
            CALL DDMSOATRV (ctot(i),csatT(i),0.0,0.0,0.0,i,2,iout)
c
c========================== DDM End ====================================
c
         else
            icont=icont+1
            idx(icont) = i
            smw(icont)=mwsoap(i)
            scsat(icont)=csatT(i)
            sctot(icont)=ctot(i)
            scaer(icont)=caer(i)
         endif
      enddo
      nsol=icont
c
c Check for a trivial solution
c
      if (nsol.eq.0) goto 1000
      if (nsol.eq.1) then
         if (cpre.lt.cpremin) then
           scgas(1) = amin1(sctot(1), scsat(1))
           scaer(1) = sctot(1) - scgas(1)
c
c========================= DDM Begin ===================================
c
           CALL DDMSOATRV (sctot(1),scsat(1),0.0,0.0,0.0,idx(1),1,iout)
c
c========================== DDM End ====================================
c
         else ! This case has an analytical solution
           bb = scsat(1)-sctot(1)+cpx*smw(1)
           cc = -sctot(1)*cpx*smw(1)
           scaer(1) = amin1( sctot(1), .5*(-bb+SQRT(bb*bb-4.*cc)) )
           scgas(1) = sctot(1) - scaer(1)
c
c========================= DDM Begin ===================================
c
           CALL DDMSOATRV (sctot(1),scsat(1),
     &                     cpre,smw(1)/mwpre,scaer(1),idx(1),3,iout)
c
c========================== DDM End ====================================
c
         endif
         goto 900
      endif
      sum=0.0
      do i=1,nsol
         sum = sum + sctot(i)/scsat(i)
      enddo
      if (cpre.lt.cpremin .and. sum.le.1.0) then
         do i=1,nsol
            scgas(i)=sctot(i)
            scaer(i)=0.0
c
c========================= DDM Begin ===================================
c
            CALL DDMSOATRV (sctot(i),scsat(i),0.0,0.0,0.0,idx(i),2,iout)
c
c========================== DDM End ====================================
c
         enddo
         goto 900
      endif
c
c Find the solution using a bi-section method (approach from max)
c
      xend = 0.0
      do i = 1, nsol
        xend = xend + sctot(i)/smw(i)
      enddo
      xend = xend + cpx
      call spfcn (nsol,sctot,scsat,scaer,smw,cpx,xend,fend)
      if ( abs(fend).le.xtola .OR.
     &    (xend-cpx).lt.cpx*1.e-3 ) goto 99
      if (fend.gt.0.0) then
        write (iout,'(//,a)') ' ERROR in SOAP:'
        write (iout,'(/,a)') ' ERROR: positive end point'
        goto 50
      endif
      dx = xend - cpx
      do znum = 1, 200
        dx = 0.5 * dx
        xmid = xend - dx
        call spfcn (nsol,sctot,scsat,scaer,smw,cpx,xmid,fmid)
        if (abs(fmid).le.xtola .or.
     &      dx.le.max(xtola,xtolr*xmid)) goto 100
        if (fmid.lt.0.0) xend = xmid
      enddo
      write (iout,'(//,a)') ' ERROR in SOAP:'
      write (iout,'(/,a)') ' ERROR: max number of iterations reached'
 50   write (iout,'(a,i10,a,3i4)') ' Grid = ', igrdchm,
     &                 ' cell(i,j,k) = ', ichm, jchm, kchm
      write (iout,'(a5,2a15)') ' spec','total [ug/m3]','c* [ug/m3]   '
      write (iout,'(i5,2e15.6)') (idx(i),sctot(i),scsat(i),i=1,nsol)
      write (iout,'(a,2e15.6)') ' cpre,cpx ',cpre,cpx
      call camxerr()
c
c Converged
c
  99  xmid = xend
 100  continue
      do i=1,nsol
         scaer(i) = amin1( sctot(i), scaer(i) )
         scgas(i) = sctot(i) - scaer(i)
      enddo
c
c========================= DDM Begin ===================================
c
      CALL DDMSOAAP (sctot,scsat,smw,mwpre,xmid,nsol,idx,iout)
c
c========================== DDM End ====================================
c
c REMAP COMPOUNDS THAT FORM SOLUTIONS BACK ONTO ORIGINAL ARRAYS
c      
 900  continue
      do i=1,nsol
         caer(idx(i))=scaer(i)
         cgas(idx(i))=scgas(i)
      enddo
 1000 continue
c
c Calculate fraction of polymerized SOA
c
      fpoly = amin1( fpoly0 * dt, 1.0 ) ! fraction of polymerization
      do i=1,NASOA                  ! anthropogenic
        apoly(i) = fpoly * caer(i)
        caer(i) = caer(i) - apoly(i)
        sopa = sopa + apoly(i)
      enddo
      do i=NASOA+1,ntot-1           ! biogenic [except SQT]
        apoly(i) = fpoly * caer(i)
        caer(i) = caer(i) - apoly(i)
        sopb = sopb + apoly(i)
      enddo
      apoly(ntot) = 0.0             ! no polymerization for SQT
c
c Convert to ppm if inputs in ppm
c
      if (lppm) then
         do i=1,ntot
            cgas(i) = cgas(i)/(convfac*mwsoap(i))
         enddo
      endif
c
      return
      end
