      subroutine vbspart(ntot,nset,caer,cgas,mwnvol,cnvol,tempk,convfac,
     &                   iout,igrdchm,ichm,jchm,kchm,lppm,
     &                   csatT)
      implicit none
c
c     VBSPART is a VBS counterpart to SOAP
c
c  MODIFICATIONS
c
c     12/07/14 -bkoo-      Modified for VBS from SOAP
c
c DEFINITION OF VARIABLES:
c
c  INPUTS
c
c     ntot    - total number of CG/SOA pairs
c     nset    - number of basis sets
c     caer    - aerosol-phase concentrations of SOA species (ug/m3)
c     cgas    - gas-phase concentrations of CG species (ppm or ug/m3)
c     mwnvol  - molecular weights of non-volatile species (for consistency check)
c     cnvol   - concentrations of non-volatile organic aerosols (ug/m3)
c     tempk   - temperature (K)
c     convfac - conversion factor: umol/m3 = ppm * convfac 
c     iout    - standard output file unit
c     igrdchm - index for grid containing the grid cell
c     ichm    - i index of grid cell
c     jchm    - j index of grid cell
c     kchm    - k index of grid cell
c     lppm    - gases in ppm if true, ug/m3 if false
c
c   OUTPUTS
c
c     caer    - aerosol-phase concentrations of SOA species (ug/m3)
c     cgas    - gas-phase concentrations of CG species (ppm or ug/m3)
c     csatT   - saturation concentrations of CG/SOA pairs at current T
c               (ug/m3)
c
c   VARIABLES USED WITHIN SOAP
c
c     nsol    - total number of solution-forming SOA species
c     cstemp0 - temperatures corresponding to saturation concentrations
c               of CG/SOA species (K)
c     csat0   - saturation concentrations of CG/SOA species (ug/m3)
c     deltah0 - enthalpy of vaporization of CG/SOA species (J/mol)
c     flagsoap- set to 1 if CG/SOA species forms solutions; 0 if not
c     mw_vbs  - molecular weights of volatile CG/SOA pairs (g/mol)
c     mw_nvol - molecular weights of non-volatile CG/SOA pairs (g/mol)
c     scaer   - aerosol-phase concentrations of solution-forming 
c               SOA species (ug/m3)
c     scgas   - gas-phase concentrations of solution-forming 
c               SOA species (ug/m3)
c     scsat   - saturation concentrations of solution-forming 
c               SOA species (ug/m3)
c     sctot   - total concentrations of solution-forming SOA species
c               (ug/m3)
c     smw     - molecular weights of solution-forming SOA species 
c               (g/mol)
c     znum    - counter for number of iterations
c     conmin  - use simple solution for species below this level (ug/m3)
c     cpremin - no pre-existing organics if cpre < cpremin (ug/m3) 
c     xtol    - error tolerance for bi-section method
c
c***********************************************************************
c
c VARIABLE DECLARATION
c
      include 'vbs.inc'
      real, parameter :: conmin  = 1.e-6
      real, parameter :: cpremin = 1.01e-9
      real, parameter :: xtola   = 1.e-8
      real, parameter :: xtolr   = 1.e-6

      integer :: ntot, nset
      real, dimension(nset) :: mwnvol, cnvol
      real, dimension(ntot) :: caer, cgas, ctot, csatT,
     &                         smw, scsat, sctot, scaer, scgas
      integer, dimension(ntot) :: idx

      real    :: cpre, tempk, convfac, sum
      integer :: iout, igrdchm, ichm, jchm, kchm
      integer :: i, ierr, icont, nsol, znum
      logical :: lppm

      real    :: cpx, bb, cc, xend, fend, xmid, fmid, dx

      logical, save :: lfirsttime = .true.
c
c***********************************************************************
c
c Entry point
c
      if (lfirsttime) then
        lfirsttime = .false.
        if(ntot.ne.nset*nvbs) then 
          write(iout,'(//,A)') 'ERROR in VBSPART:'
          write(iout,*) 'Parameter mismatch'
          write(iout,'(1X,A,I2)') 'VBS.INC: NVBS = ',nvbs
          write(iout,'(1X,A,I2)') 'Argument: NTOT, NSET = ',ntot,nset
          write(iout,*) 'Please set the parameter to be consistent.'
          call camxerr()
        endif
        ierr = 0
        do i = 1, nset
          if (mwnvol(i).ne.MW_NVOL(i)) ierr = 1
        enddo
        if (ierr.gt.0) then
          write(iout,'(//,A)') 'ERROR in VBSPART:'
          write(iout,'(1X,A)') 'Inconsistent NVOL MWs'
          write(iout,'(2X,A,F10.4)') 'VBS.INC  = ',(MW_NVOL(i),i=1,nset)
          write(iout,'(2X,A,F10.4)') 'Argument = ',(mwnvol(i),i=1,nset)
          call camxerr()
        endif
      endif

      do i=1,ntot
        if (lppm) cgas(i) = cgas(i) * convfac * mw_vbs(i) ! convert to ug/m3
        ctot(i) = caer(i) + cgas(i)
      enddo
c
c Set pre-existing (non-volatile) organic mass
c
      cpre = 0.0
      cpx  = 0.0
      do i=1,nset
        cpre = cpre + cnvol(i)
        cpx  = cpx  + cnvol(i) / mw_nvol(i)
      enddo
c
c CHANGE SATURATION CONCENTRATIONS ACCORDING TO CURRENT TEMPERATURE
c
      do i=1,ntot
         csatT(i)=csat0(i)*(cstemp0(i)/tempk)*exp((deltah0(i)/8.314)
     &                *(1/cstemp0(i)-1/tempk))
      enddo     
c
c CALCULATE AEROSOL-PHASE CONCENTRATION (CAER) AND GAS-PHASE 
c CONCENTRATION (CGAS) FOR NON-SOLUTION-FORMING COMPOUNDS
c COMPOUNDS THAT HAVE A CONCENTRATION OF LESS THAN conmin ARE IGN0RED
c MAP COMPOUNDS THAT FORM SOLUTIONS ONTO ARRAYS
c
      icont=0
      do i=1,ntot
cbk         if (flagsoap(i).eq.0) then
cbk            cgas(i) = amin1(ctot(i), csatT(i))
cbk            caer(i) = ctot(i) - cgas(i)
cbk         elseif (ctot(i).lt.conmin) then
         if (ctot(i).lt.conmin) then
            cgas(i) = ctot(i)
            caer(i) = 0.0
         else
            icont=icont+1
            idx(icont) = i
            smw(icont)=mw_vbs(i)
            scsat(icont)=csatT(i)
            sctot(icont)=ctot(i)
            scaer(icont)=caer(i)
         endif
      enddo
      nsol=icont
c
c Check for a trivial solution
c
      if (nsol.eq.0) goto 1000
      if (nsol.eq.1) then
         if (cpre.lt.cpremin) then
           scgas(1) = amin1(sctot(1), scsat(1))
           scaer(1) = sctot(1) - scgas(1)
         else ! This case has an analytical solution
           bb = scsat(1)-sctot(1)+cpx*smw(1)
           cc = -sctot(1)*cpx*smw(1)
           scaer(1) = amin1( sctot(1), .5*(-bb+SQRT(bb*bb-4.*cc)) )
           scgas(1) = sctot(1) - scaer(1)
         endif
         goto 900
      endif
      sum=0.0
      do i=1,nsol
         sum = sum + sctot(i)/scsat(i)
      enddo
      if (cpre.lt.cpremin .and. sum.le.1.0) then
         do i=1,nsol
            scgas(i)=sctot(i)
            scaer(i)=0.0
         enddo
         goto 900
      endif
c
c Find the solution using a bi-section method (approach from max)
c
      xend = 0.0
      do i = 1, nsol
        xend = xend + sctot(i)/smw(i)
      enddo
      xend = xend + cpx
      call spfcn (nsol,sctot,scsat,scaer,smw,cpx,xend,fend)
      if ( abs(fend).le.xtola .OR.
     &    (xend-cpx).lt.cpx*1.e-3 ) goto 99
      if (fend.gt.0.0) then
        write (iout,'(//,a)') ' ERROR in VBSPART:'
        write (iout,'(/,a)') ' ERROR: positive end point'
        goto 50
      endif
      dx = xend - cpx
      do znum = 1, 200
        dx = 0.5 * dx
        xmid = xend - dx
        call spfcn (nsol,sctot,scsat,scaer,smw,cpx,xmid,fmid)
        if (abs(fmid).le.xtola .or.
     &      dx.le.max(xtola,xtolr*xmid)) goto 100
        if (fmid.lt.0.0) xend = xmid
      enddo
      write (iout,'(//,a)') ' ERROR in VBSPART:'
      write (iout,'(/,a)') ' ERROR: max number of iterations reached'
 50   write (iout,'(a,i10,a,3i4)') ' Grid = ', igrdchm,
     &                 ' cell(i,j,k) = ', ichm, jchm, kchm
      write (iout,'(a5,2a15)') ' spec','total [ug/m3]','c* [ug/m3]   '
      write (iout,'(i5,2e15.6)') (idx(i),sctot(i),scsat(i),i=1,nsol)
      write (iout,'(a,2e15.6)') ' cpre,cpx ',cpre,cpx
      call camxerr()
c
c Converged
c
  99  xmid = xend
 100  continue
      do i=1,nsol
         scaer(i) = amin1( sctot(i), scaer(i) )
         scgas(i) = sctot(i) - scaer(i)
      enddo
c
c REMAP COMPOUNDS THAT FORM SOLUTIONS BACK ONTO ORIGINAL ARRAYS
c      
 900  continue
      do i=1,nsol
         caer(idx(i))=scaer(i)
         cgas(idx(i))=scgas(i)
      enddo
c
c Convert to ppm if inputs in ppm
c
 1000 continue
      if (lppm) then
         do i=1,ntot
            cgas(i) = cgas(i) / (convfac * mw_vbs(i))
         enddo
      endif
c
      return
      end
